"use strict";
var Connection_1 = require("../connection/Connection");
var ConnectionNotFoundError_1 = require("./error/ConnectionNotFoundError");
var MysqlDriver_1 = require("../driver/MysqlDriver");
var MissingDriverError_1 = require("./error/MissingDriverError");
/**
 * Connection manager holds all connections made to the databases and providers helper management functions
 * for all exist connections.
 */
var ConnectionManager = (function () {
    function ConnectionManager() {
        // -------------------------------------------------------------------------
        // Properties
        // -------------------------------------------------------------------------
        this.connections = [];
    }
    // -------------------------------------------------------------------------
    // Public Methods
    // -------------------------------------------------------------------------
    /**
     * Creates a new connection based on the given connection options and registers a new connection in the manager.
     */
    ConnectionManager.prototype.create = function (options) {
        var driver = this.createDriver(options.driver);
        var connection = this.createConnection(options.connectionName || "default", driver, options.connection);
        if (options.entityDirectories && options.entityDirectories.length > 0)
            connection.importEntitiesFromDirectories(options.entityDirectories);
        if (options.entities)
            connection.importEntities(options.entities);
        if (options.subscriberDirectories && options.subscriberDirectories.length > 0)
            connection.importSubscribersFromDirectories(options.subscriberDirectories);
        if (options.subscribers)
            connection.importSubscribers(options.subscribers);
        if (options.namingStrategyDirectories && options.namingStrategyDirectories.length > 0)
            connection.importNamingStrategiesFromDirectories(options.namingStrategyDirectories);
        if (options.namingStrategies)
            connection.importNamingStrategies(options.namingStrategies);
        return connection.connect();
    };
    /**
     * Gets registered connection with the given name. If connection name is not given then it will get a default
     * connection.
     */
    ConnectionManager.prototype.get = function (name) {
        if (name === void 0) { name = "default"; }
        if (!name)
            name = "default";
        var connection = this.connections.find(function (connection) { return connection.name === name; });
        if (!connection)
            throw new ConnectionNotFoundError_1.ConnectionNotFoundError(name);
        return connection;
    };
    // -------------------------------------------------------------------------
    // Private Methods
    // -------------------------------------------------------------------------
    ConnectionManager.prototype.createDriver = function (driverName) {
        switch (driverName) {
            case "mysql":
                return new MysqlDriver_1.MysqlDriver();
            default:
                throw new MissingDriverError_1.MissingDriverError(driverName);
        }
    };
    /**
     * Creates a new connection and pushes a connection to the array.
     */
    ConnectionManager.prototype.createConnection = function (name, driver, options) {
        var existConnection = this.connections.find(function (connection) { return connection.name === name; });
        if (existConnection)
            this.connections.splice(this.connections.indexOf(existConnection), 1);
        var connection = new Connection_1.Connection(name, driver, options);
        this.connections.push(connection);
        return connection;
    };
    return ConnectionManager;
}());
exports.ConnectionManager = ConnectionManager;
//# sourceMappingURL=ConnectionManager.js.map