import { Driver } from "../driver/Driver";
import { ConnectionOptions } from "./ConnectionOptions";
import { Repository } from "../repository/Repository";
import { EventSubscriberInterface } from "../subscriber/EventSubscriberInterface";
import { ConstructorFunction } from "../common/ConstructorFunction";
import { EntityListenerMetadata } from "../metadata/EntityListenerMetadata";
import { EntityManager } from "../entity-manager/EntityManager";
import { ReactiveRepository } from "../repository/ReactiveRepository";
import { ReactiveEntityManager } from "../entity-manager/ReactiveEntityManager";
import { TreeRepository } from "../repository/TreeRepository";
import { ReactiveTreeRepository } from "../repository/ReactiveTreeRepository";
/**
 * A single connection instance to the database. Each connection has its own repositories, subscribers and metadatas.
 */
export declare class Connection {
    private repositoryAndMetadatas;
    /**
     * Gets EntityManager of this connection.
     */
    readonly entityManager: EntityManager;
    /**
     * Gets ReactiveEntityManager of this connection.
     */
    readonly reactiveEntityManager: ReactiveEntityManager;
    /**
     * The name of the connection.
     */
    readonly name: string;
    /**
     * Database driver used by this connection.
     */
    readonly driver: Driver;
    /**
     * All entity listener metadatas that are registered for this connection.
     */
    readonly entityListeners: EntityListenerMetadata[];
    /**
     * All subscribers that are registered for this connection.
     */
    readonly eventSubscribers: EventSubscriberInterface<any>[];
    /**
     * Connection options.
     */
    private readonly options;
    /**
     * All entity metadatas that are registered for this connection.
     */
    private readonly entityMetadatas;
    /**
     * All naming strategy metadatas that are registered for this connection.
     */
    private readonly namingStrategyMetadatas;
    /**
     * Registered entity classes to be used for this connection.
     */
    private readonly entityClasses;
    /**
     * Registered subscriber classes to be used for this connection.
     */
    private readonly subscriberClasses;
    /**
     * Registered naming strategy classes to be used for this connection.
     */
    private readonly namingStrategyClasses;
    /**
     * Indicates if connection has been done or not.
     */
    private _isConnected;
    constructor(name: string, driver: Driver, options: ConnectionOptions);
    /**
     * Returns true if connection to the database already established for this connection.
     */
    readonly isConnected: boolean;
    /**
     * Performs connection to the database.
     */
    connect(): Promise<this>;
    /**
     * Closes this connection.
     */
    close(): Promise<void>;
    /**
     * Creates database schema for all entities registered in this connection.
     */
    syncSchema(): Promise<void>;
    /**
     * Imports entities from the given paths (directories) for the current connection.
     */
    importEntitiesFromDirectories(paths: string[]): this;
    /**
     * Imports subscribers from the given paths (directories) for the current connection.
     */
    importSubscribersFromDirectories(paths: string[]): this;
    /**
     * Imports naming strategies from the given paths (directories) for the current connection.
     */
    importNamingStrategiesFromDirectories(paths: string[]): this;
    /**
     * Imports entities for the current connection.
     */
    importEntities(entities: Function[]): this;
    /**
     * Imports entities for the given connection. If connection name is not given then default connection is used.
     */
    importSubscribers(subscriberClasses: Function[]): this;
    /**
     * Imports entities for the current connection.
     */
    importNamingStrategies(strategies: Function[]): this;
    /**
     * Gets repository for the given entity class.
     */
    getRepository<Entity>(entityClass: ConstructorFunction<Entity> | Function): Repository<Entity>;
    /**
     * Gets repository for the given entity name.
     */
    getRepository<Entity>(entityClass: string): Repository<Entity>;
    /**
     * Gets repository for the given entity class.
     */
    getTreeRepository<Entity>(entityClass: ConstructorFunction<Entity> | Function): TreeRepository<Entity>;
    /**
     * Gets repository for the given entity name.
     */
    getTreeRepository<Entity>(entityClass: string): TreeRepository<Entity>;
    /**
     * Gets reactive repository for the given entity class.
     */
    getReactiveRepository<Entity>(entityClass: ConstructorFunction<Entity> | Function): ReactiveRepository<Entity>;
    /**
     * Gets reactive tree repository for the given entity class.
     */
    getReactiveTreeRepository<Entity>(entityClass: ConstructorFunction<Entity> | Function): ReactiveTreeRepository<Entity>;
    /**
     * Builds all registered metadatas.
     */
    private buildMetadatas();
    /**
     * Gets the naming strategy to be used for this connection.
     */
    private createNamingStrategy();
    /**
     * Creates a new instance of the given constructor. If service container is registered in the ORM, then it will
     * be used, otherwise new instance of naming strategy will be created.
     */
    private createContainerInstance(constructor);
    /**
     * Creates a temporary object RepositoryAndMetadata to store relation between repository and metadata.
     */
    private createRepoMeta(metadata);
}
