"use strict";
var Repository_1 = require("../repository/Repository");
var RepositoryNotFoundError_1 = require("./error/RepositoryNotFoundError");
var SchemaCreator_1 = require("../schema-creator/SchemaCreator");
var EntityManager_1 = require("../entity-manager/EntityManager");
var DirectoryExportedClassesLoader_1 = require("../util/DirectoryExportedClassesLoader");
var index_1 = require("../index");
var EntityMetadataBuilder_1 = require("../metadata-storage/EntityMetadataBuilder");
var DefaultNamingStrategy_1 = require("../naming-strategy/DefaultNamingStrategy");
var EntityMetadataCollection_1 = require("../metadata/collection/EntityMetadataCollection");
var NoConnectionForRepositoryError_1 = require("./error/NoConnectionForRepositoryError");
var CannotImportAlreadyConnectedError_1 = require("./error/CannotImportAlreadyConnectedError");
var CannotCloseNotConnectedError_1 = require("./error/CannotCloseNotConnectedError");
var CannotConnectAlreadyConnectedError_1 = require("./error/CannotConnectAlreadyConnectedError");
var ReactiveRepository_1 = require("../repository/ReactiveRepository");
var ReactiveEntityManager_1 = require("../entity-manager/ReactiveEntityManager");
var TreeRepository_1 = require("../repository/TreeRepository");
var ReactiveTreeRepository_1 = require("../repository/ReactiveTreeRepository");
/**
 * A single connection instance to the database. Each connection has its own repositories, subscribers and metadatas.
 */
var Connection = (function () {
    // -------------------------------------------------------------------------
    // Constructor
    // -------------------------------------------------------------------------
    function Connection(name, driver, options) {
        // -------------------------------------------------------------------------
        // Properties
        // -------------------------------------------------------------------------
        this.repositoryAndMetadatas = [];
        /**
         * All entity listener metadatas that are registered for this connection.
         */
        this.entityListeners = [];
        /**
         * All subscribers that are registered for this connection.
         */
        this.eventSubscribers = [];
        /**
         * All entity metadatas that are registered for this connection.
         */
        this.entityMetadatas = new EntityMetadataCollection_1.EntityMetadataCollection();
        /**
         * All naming strategy metadatas that are registered for this connection.
         */
        this.namingStrategyMetadatas = [];
        /**
         * Registered entity classes to be used for this connection.
         */
        this.entityClasses = [];
        /**
         * Registered subscriber classes to be used for this connection.
         */
        this.subscriberClasses = [];
        /**
         * Registered naming strategy classes to be used for this connection.
         */
        this.namingStrategyClasses = [];
        /**
         * Indicates if connection has been done or not.
         */
        this._isConnected = false;
        this.name = name;
        this.driver = driver;
        this.driver.connectionOptions = options;
        this.options = options;
        this.entityManager = new EntityManager_1.EntityManager(this);
        this.reactiveEntityManager = new ReactiveEntityManager_1.ReactiveEntityManager(this);
    }
    Object.defineProperty(Connection.prototype, "isConnected", {
        // -------------------------------------------------------------------------
        // Accessors
        // -------------------------------------------------------------------------
        /**
         * Returns true if connection to the database already established for this connection.
         */
        get: function () {
            return this._isConnected;
        },
        enumerable: true,
        configurable: true
    });
    // -------------------------------------------------------------------------
    // Public Methods
    // -------------------------------------------------------------------------
    /**
     * Performs connection to the database.
     */
    Connection.prototype.connect = function () {
        var _this = this;
        if (this.isConnected)
            throw new CannotConnectAlreadyConnectedError_1.CannotConnectAlreadyConnectedError(this.name);
        return this.driver.connect().then(function () {
            // first build all metadata
            _this.buildMetadatas();
            // second build schema
            if (_this.options.autoSchemaCreate === true)
                return _this.syncSchema();
            return Promise.resolve();
        }).then(function () {
            _this._isConnected = true;
            return _this;
        });
    };
    /**
     * Closes this connection.
     */
    Connection.prototype.close = function () {
        if (!this.isConnected)
            throw new CannotCloseNotConnectedError_1.CannotCloseNotConnectedError(this.name);
        return this.driver.disconnect();
    };
    /**
     * Creates database schema for all entities registered in this connection.
     */
    Connection.prototype.syncSchema = function () {
        var schemaBuilder = this.driver.createSchemaBuilder();
        var schemaCreator = new SchemaCreator_1.SchemaCreator(schemaBuilder, this.entityMetadatas);
        return schemaCreator.create();
    };
    /**
     * Imports entities from the given paths (directories) for the current connection.
     */
    Connection.prototype.importEntitiesFromDirectories = function (paths) {
        this.importEntities(DirectoryExportedClassesLoader_1.importClassesFromDirectories(paths));
        return this;
    };
    /**
     * Imports subscribers from the given paths (directories) for the current connection.
     */
    Connection.prototype.importSubscribersFromDirectories = function (paths) {
        this.importSubscribers(DirectoryExportedClassesLoader_1.importClassesFromDirectories(paths));
        return this;
    };
    /**
     * Imports naming strategies from the given paths (directories) for the current connection.
     */
    Connection.prototype.importNamingStrategiesFromDirectories = function (paths) {
        this.importEntities(DirectoryExportedClassesLoader_1.importClassesFromDirectories(paths));
        return this;
    };
    /**
     * Imports entities for the current connection.
     */
    Connection.prototype.importEntities = function (entities) {
        var _this = this;
        if (this.isConnected)
            throw new CannotImportAlreadyConnectedError_1.CannotImportAlreadyConnectedError("entities", this.name);
        entities.forEach(function (cls) { return _this.entityClasses.push(cls); });
        return this;
    };
    /**
     * Imports entities for the given connection. If connection name is not given then default connection is used.
     */
    Connection.prototype.importSubscribers = function (subscriberClasses) {
        var _this = this;
        if (this.isConnected)
            throw new CannotImportAlreadyConnectedError_1.CannotImportAlreadyConnectedError("event subscribers", this.name);
        subscriberClasses.forEach(function (cls) { return _this.subscriberClasses.push(cls); });
        return this;
    };
    /**
     * Imports entities for the current connection.
     */
    Connection.prototype.importNamingStrategies = function (strategies) {
        var _this = this;
        if (this.isConnected)
            throw new CannotImportAlreadyConnectedError_1.CannotImportAlreadyConnectedError("naming strategies", this.name);
        strategies.forEach(function (cls) { return _this.namingStrategyClasses.push(cls); });
        return this;
    };
    /**
     * Gets repository for the given entity class or name.
     */
    Connection.prototype.getRepository = function (entityClassOrName) {
        if (!this.isConnected)
            throw new NoConnectionForRepositoryError_1.NoConnectionForRepositoryError(this.name);
        var metadata;
        if (typeof entityClassOrName === "string") {
            metadata = this.entityMetadatas.findByName(entityClassOrName);
        }
        else {
            metadata = this.entityMetadatas.findByTarget(entityClassOrName);
        }
        var repoMeta = this.repositoryAndMetadatas.find(function (repoMeta) { return repoMeta.metadata === metadata; });
        if (!repoMeta)
            throw new RepositoryNotFoundError_1.RepositoryNotFoundError(this.name, entityClassOrName);
        return repoMeta.repository;
    };
    /**
     * Gets repository for the given entity class or name.
     */
    Connection.prototype.getTreeRepository = function (entityClassOrName) {
        if (!this.isConnected)
            throw new NoConnectionForRepositoryError_1.NoConnectionForRepositoryError(this.name);
        var metadata;
        if (typeof entityClassOrName === "string") {
            metadata = this.entityMetadatas.findByName(entityClassOrName);
        }
        else {
            metadata = this.entityMetadatas.findByTarget(entityClassOrName);
        }
        var repoMeta = this.repositoryAndMetadatas.find(function (repoMeta) { return repoMeta.metadata === metadata; });
        if (!repoMeta)
            throw new RepositoryNotFoundError_1.RepositoryNotFoundError(this.name, entityClassOrName);
        if (!repoMeta.metadata.table.isClosure)
            throw new Error("Cannot get a tree repository. " + repoMeta.metadata.name + " is not a tree table. Try to use @ClosureTable decorator instead of @Table.");
        return repoMeta.repository;
    };
    /**
     * Gets reactive repository for the given entity class.
     */
    Connection.prototype.getReactiveRepository = function (entityClass) {
        if (!this.isConnected)
            throw new NoConnectionForRepositoryError_1.NoConnectionForRepositoryError(this.name);
        var metadata = this.entityMetadatas.findByTarget(entityClass);
        var repoMeta = this.repositoryAndMetadatas.find(function (repoMeta) { return repoMeta.metadata === metadata; });
        if (!repoMeta)
            throw new RepositoryNotFoundError_1.RepositoryNotFoundError(this.name, entityClass);
        return repoMeta.reactiveRepository;
    };
    /**
     * Gets reactive tree repository for the given entity class.
     */
    Connection.prototype.getReactiveTreeRepository = function (entityClass) {
        if (!this.isConnected)
            throw new NoConnectionForRepositoryError_1.NoConnectionForRepositoryError(this.name);
        var metadata = this.entityMetadatas.findByTarget(entityClass);
        var repoMeta = this.repositoryAndMetadatas.find(function (repoMeta) { return repoMeta.metadata === metadata; });
        if (!repoMeta)
            throw new RepositoryNotFoundError_1.RepositoryNotFoundError(this.name, entityClass);
        if (!repoMeta.metadata.table.isClosure)
            throw new Error("Cannot get a tree repository. " + repoMeta.metadata.name + " is not a tree table. Try to use @ClosureTable decorator instead of @Table.");
        return repoMeta.reactiveRepository;
    };
    // -------------------------------------------------------------------------
    // Private Methods
    // -------------------------------------------------------------------------
    /**
     * Builds all registered metadatas.
     */
    Connection.prototype.buildMetadatas = function () {
        var _this = this;
        // first register naming strategies
        var metadatas = index_1.defaultMetadataStorage().namingStrategyMetadatas.filterByClasses(this.namingStrategyClasses);
        metadatas.forEach(function (cls) { return _this.namingStrategyMetadatas.push(cls); });
        // second register subscriber metadatas
        var subscribers = index_1.defaultMetadataStorage()
            .eventSubscriberMetadatas
            .filterByClasses(this.subscriberClasses)
            .map(function (metadata) { return _this.createContainerInstance(metadata.target); });
        (_a = this.eventSubscribers).push.apply(_a, subscribers);
        // third register entity and entity listener metadatas
        var entityMetadataBuilder = new EntityMetadataBuilder_1.EntityMetadataBuilder(this.createNamingStrategy());
        var entityMetadatas = entityMetadataBuilder.build(this.entityClasses);
        var entityListenerMetadatas = index_1.defaultMetadataStorage().entityListenerMetadatas.filterByClasses(this.entityClasses);
        entityMetadatas.forEach(function (cls) { return _this.entityMetadatas.push(cls); });
        entityListenerMetadatas.forEach(function (cls) { return _this.entityListeners.push(cls); });
        entityMetadatas.map(function (metadata) { return _this.createRepoMeta(metadata); }).forEach(function (cls) { return _this.repositoryAndMetadatas.push(cls); });
        var _a;
    };
    /**
     * Gets the naming strategy to be used for this connection.
     */
    Connection.prototype.createNamingStrategy = function () {
        var _this = this;
        if (!this.options.namingStrategy)
            return new DefaultNamingStrategy_1.DefaultNamingStrategy();
        var namingMetadata = this.namingStrategyMetadatas.find(function (strategy) { return strategy.name === _this.options.namingStrategy; });
        if (!namingMetadata)
            throw new Error("Naming strategy called \"" + this.options.namingStrategy + "\" was not found.");
        return this.createContainerInstance(namingMetadata.target);
    };
    /**
     * Creates a new instance of the given constructor. If service container is registered in the ORM, then it will
     * be used, otherwise new instance of naming strategy will be created.
     */
    Connection.prototype.createContainerInstance = function (constructor) {
        return index_1.getContainer() ? index_1.getContainer().get(constructor) : new constructor();
    };
    /**
     * Creates a temporary object RepositoryAndMetadata to store relation between repository and metadata.
     */
    Connection.prototype.createRepoMeta = function (metadata) {
        if (metadata.table.isClosure) {
            var repository = new TreeRepository_1.TreeRepository(this, this.entityMetadatas, metadata);
            return {
                metadata: metadata,
                repository: repository,
                reactiveRepository: new ReactiveTreeRepository_1.ReactiveTreeRepository(repository)
            };
        }
        else {
            var repository = new Repository_1.Repository(this, this.entityMetadatas, metadata);
            return {
                metadata: metadata,
                repository: repository,
                reactiveRepository: new ReactiveRepository_1.ReactiveRepository(repository)
            };
        }
    };
    return Connection;
}());
exports.Connection = Connection;
//# sourceMappingURL=Connection.js.map