"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var QueryBuilder_1 = require("../query-builder/QueryBuilder");
var MysqlSchemaBuilder_1 = require("../schema-builder/MysqlSchemaBuilder");
var ConnectionIsNotSetError_1 = require("./error/ConnectionIsNotSetError");
var BaseDriver_1 = require("./BaseDriver");
var ColumnTypes_1 = require("../metadata/types/ColumnTypes");
var moment = require("moment");
/**
 * This driver organizes work with mysql database.
 */
var MysqlDriver = (function (_super) {
    __extends(MysqlDriver, _super);
    // -------------------------------------------------------------------------
    // Constructor
    // -------------------------------------------------------------------------
    function MysqlDriver(mysql) {
        _super.call(this);
        // if driver dependency is not given explicitly, then try to load it via "require"
        if (!mysql && require) {
            try {
                mysql = require("mysql");
            }
            catch (e) {
                throw new Error("Mysql package was not found installed. Try to install it: npm install mysql --save");
            }
        }
        else {
            throw new Error("Cannot load mysql driver dependencies. Try to install all required dependencies.");
        }
        this.mysql = mysql;
    }
    Object.defineProperty(MysqlDriver.prototype, "native", {
        // -------------------------------------------------------------------------
        // Getter Methods
        // -------------------------------------------------------------------------
        /**
         * Access to the native implementation of the database.
         */
        get: function () {
            return this.mysql;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MysqlDriver.prototype, "nativeConnection", {
        /**
         * Access to the native implementation of the database.
         */
        get: function () {
            return this.mysqlConnection;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MysqlDriver.prototype, "db", {
        /**
         * Database name to which this connection is made.
         */
        get: function () {
            if (this.mysqlConnection && this.mysqlConnection.config.database)
                return this.mysqlConnection.config.database;
            if (this.connectionOptions.database)
                return this.connectionOptions.database;
            throw new Error("Cannot get the database name. Since database name is not explicitly given in configuration " +
                "(maybe connection url is used?), database name cannot be retrieved until connection is made.");
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MysqlDriver.prototype, "queryBuilderClass", {
        // -------------------------------------------------------------------------
        // Public Methods
        // -------------------------------------------------------------------------
        /**
         * Creates a query builder which can be used to build an sql queries.
         */
        get: function () {
            return QueryBuilder_1.QueryBuilder;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * Creates a schema builder which can be used to build database/table schemas.
     */
    MysqlDriver.prototype.createSchemaBuilder = function () {
        return new MysqlSchemaBuilder_1.MysqlSchemaBuilder(this);
    };
    /**
     * Performs connection to the database based on given connection options.
     */
    MysqlDriver.prototype.connect = function () {
        var _this = this;
        this.mysqlConnection = this.mysql.createConnection({
            host: this.connectionOptions.host,
            user: this.connectionOptions.username,
            password: this.connectionOptions.password,
            database: this.connectionOptions.database
        });
        return new Promise(function (ok, fail) {
            _this.mysqlConnection.connect(function (err) { return err ? fail(err) : ok(); });
        });
    };
    /**
     * Closes connection with database.
     */
    MysqlDriver.prototype.disconnect = function () {
        var _this = this;
        if (!this.mysqlConnection)
            throw new ConnectionIsNotSetError_1.ConnectionIsNotSetError("mysql");
        return new Promise(function (ok, fail) {
            _this.mysqlConnection.end(function (err) { return err ? fail(err) : ok(); });
        });
    };
    /**
     * Executes a given SQL query.
     */
    MysqlDriver.prototype.query = function (query) {
        var _this = this;
        if (!this.mysqlConnection)
            throw new ConnectionIsNotSetError_1.ConnectionIsNotSetError("mysql");
        this.logQuery(query);
        return new Promise(function (ok, fail) { return _this.mysqlConnection.query(query, function (err, result) {
            if (err) {
                _this.logFailedQuery(query);
                _this.logQueryError(err);
                fail(err);
            }
            else {
                ok(result);
            }
        }); });
    };
    /**
     * Clears all tables in the currently connected database.
     */
    MysqlDriver.prototype.clearDatabase = function () {
        var _this = this;
        if (!this.mysqlConnection)
            throw new ConnectionIsNotSetError_1.ConnectionIsNotSetError("mysql");
        var disableForeignKeysCheckQuery = "SET FOREIGN_KEY_CHECKS = 0;";
        var dropTablesQuery = "SELECT concat('DROP TABLE IF EXISTS ', table_name, ';') AS q FROM " +
            ("information_schema.tables WHERE table_schema = '" + this.db + "';");
        var enableForeignKeysCheckQuery = "SET FOREIGN_KEY_CHECKS = 1;";
        return this.query(disableForeignKeysCheckQuery)
            .then(function () { return _this.query(dropTablesQuery); })
            .then(function (results) { return Promise.all(results.map(function (q) { return _this.query(q["q"]); })); })
            .then(function () { return _this.query(enableForeignKeysCheckQuery); })
            .then(function () { });
    };
    /**
     * Updates rows that match given conditions in the given table.
     */
    MysqlDriver.prototype.update = function (tableName, valuesMap, conditions) {
        if (!this.mysqlConnection)
            throw new ConnectionIsNotSetError_1.ConnectionIsNotSetError("mysql");
        var updateValues = this.escapeObjectMap(valuesMap).join(",");
        var conditionString = this.escapeObjectMap(conditions).join(" AND ");
        var query = "UPDATE " + tableName + " SET " + updateValues + " " + (conditionString ? (" WHERE " + conditionString) : "");
        // console.log("executing update: ", query);
        return this.query(query).then(function () { });
        // const qb = this.createQueryBuilder().update(tableName, valuesMap).from(tableName, "t");
        // Object.keys(conditions).forEach(key => qb.andWhere(key + "=:" + key, { [key]: (<any> conditions)[key] }));
        // return qb.execute().then(() => {});
    };
    MysqlDriver.prototype.escapeObjectMap = function (objectMap) {
        var _this = this;
        return Object.keys(objectMap).map(function (key) {
            var value = objectMap[key];
            if (value === null || value === undefined) {
                return key + "=NULL";
            }
            else {
                return key + "=" + _this.escape(value);
            }
        });
    };
    /**
     * Insert a new row into given table.
     */
    MysqlDriver.prototype.insert = function (tableName, keyValues) {
        var _this = this;
        if (!this.mysqlConnection)
            throw new ConnectionIsNotSetError_1.ConnectionIsNotSetError("mysql");
        var columns = Object.keys(keyValues).join(",");
        // const values  = this.escapeObjectMap(keyValues).join(",");
        var values = Object.keys(keyValues).map(function (key) { return _this.escape(keyValues[key]); }).join(","); // todo: escape here
        var query = "INSERT INTO " + tableName + "(" + columns + ") VALUES (" + values + ")";
        return this.query(query).then(function (result) { return result.insertId; });
    };
    /**
     * Deletes from the given table by a given conditions.
     */
    MysqlDriver.prototype.delete = function (tableName, conditions) {
        if (!this.mysqlConnection)
            throw new ConnectionIsNotSetError_1.ConnectionIsNotSetError("mysql");
        var conditionString = this.escapeObjectMap(conditions).join(" AND ");
        var query = "DELETE FROM " + tableName + " WHERE " + conditionString;
        return this.query(query).then(function () { });
        // const qb = this.createQueryBuilder().delete(tableName);
        // Object.keys(conditions).forEach(key => qb.andWhere(key + "=:" + key, { [key]: (<any> conditions)[key] }));
        // return qb.execute().then(() => {});
    };
    /**
     * Starts mysql transaction.
     */
    MysqlDriver.prototype.beginTransaction = function () {
        return this.query("START TRANSACTION").then(function () { });
    };
    /**
     * Ends mysql transaction.
     */
    MysqlDriver.prototype.endTransaction = function () {
        return this.query("COMMIT").then(function () { });
    };
    /**
     * Prepares given value to a value to be persisted, based on its column type and metadata.
     */
    MysqlDriver.prototype.preparePersistentValue = function (value, column) {
        switch (column.type) {
            case ColumnTypes_1.ColumnTypes.BOOLEAN:
                return value === true ? 1 : 0;
            case ColumnTypes_1.ColumnTypes.DATE:
                return moment(value).format("YYYY-MM-DD");
            case ColumnTypes_1.ColumnTypes.TIME:
                return moment(value).format("hh:mm:ss");
            case ColumnTypes_1.ColumnTypes.DATETIME:
                return moment(value).format("YYYY-MM-DD hh:mm:ss");
            case ColumnTypes_1.ColumnTypes.JSON:
                return JSON.stringify(value);
            case ColumnTypes_1.ColumnTypes.SIMPLE_ARRAY:
                return value
                    .map(function (i) { return String(i); })
                    .join(",");
        }
        return value;
    };
    /**
     * Prepares given value to a value to be persisted, based on its column type and metadata.
     */
    MysqlDriver.prototype.prepareHydratedValue = function (value, column) {
        switch (column.type) {
            case ColumnTypes_1.ColumnTypes.DATE:
                if (value instanceof Date)
                    return value;
                return moment(value, "YYYY-MM-DD").toDate();
            case ColumnTypes_1.ColumnTypes.TIME:
                return moment(value, "hh:mm:ss").toDate();
            case ColumnTypes_1.ColumnTypes.DATETIME:
                if (value instanceof Date)
                    return value;
                return moment(value, "YYYY-MM-DD hh:mm:ss").toDate();
            case ColumnTypes_1.ColumnTypes.JSON:
                return JSON.parse(value);
            case ColumnTypes_1.ColumnTypes.SIMPLE_ARRAY:
                return value.split(",");
        }
        return value;
    };
    /**
     * Escapes given value.
     */
    MysqlDriver.prototype.escape = function (value) {
        return this.mysqlConnection.escape(value);
    };
    /**
     * Inserts rows into closure table.
     */
    MysqlDriver.prototype.insertIntoClosureTable = function (tableName, newEntityId, parentId, hasLevel) {
        var _this = this;
        var sql = "";
        if (hasLevel) {
            sql = ("INSERT INTO " + tableName + "(ancestor, descendant, level) ") +
                ("SELECT ancestor, " + newEntityId + ", level + 1 FROM " + tableName + " WHERE descendant = " + parentId + " ") +
                ("UNION ALL SELECT " + newEntityId + ", " + newEntityId + ", 1");
        }
        else {
            sql = ("INSERT INTO " + tableName + "(ancestor, descendant) ") +
                ("SELECT ancestor, " + newEntityId + " FROM " + tableName + " WHERE descendant = " + parentId + " ") +
                ("UNION ALL SELECT " + newEntityId + ", " + newEntityId);
        }
        return this.query(sql).then(function () {
            return _this.query("SELECT MAX(level) as level FROM " + tableName + " WHERE descendant = " + parentId);
        }).then(function (results) {
            return results && results[0] && results[0]["level"] ? parseInt(results[0]["level"]) + 1 : 1;
        });
    };
    return MysqlDriver;
}(BaseDriver_1.BaseDriver));
exports.MysqlDriver = MysqlDriver;
//# sourceMappingURL=MysqlDriver.js.map