"use strict";
/**
 * Entity manager supposed to work with any entity, automatically find its repository and call its method, whatever
 * entity type are you passing.
 */
var EntityManager = (function () {
    // -------------------------------------------------------------------------
    // Constructor
    // -------------------------------------------------------------------------
    function EntityManager(connection) {
        this.connection = connection;
    }
    /**
     * Gets repository for the given entity class or name.
     */
    EntityManager.prototype.getRepository = function (entityClassOrName) {
        if (typeof entityClassOrName === "string") {
            return this.connection.getRepository(entityClassOrName);
        }
        else {
            return this.connection.getRepository(entityClassOrName);
        }
    };
    /**
     * Gets repository for the given entity class or name.
     */
    EntityManager.prototype.getTreeRepository = function (entityClassOrName) {
        if (typeof entityClassOrName === "string") {
            return this.connection.getTreeRepository(entityClassOrName);
        }
        else {
            return this.connection.getTreeRepository(entityClassOrName);
        }
    };
    /**
     * Gets reactive repository of the given entity.
     */
    EntityManager.prototype.getReactiveRepository = function (entityClass) {
        return this.connection.getReactiveRepository(entityClass);
    };
    /**
     * Checks if entity has an id.
     */
    EntityManager.prototype.hasId = function (entity) {
        return this.getRepository(entity.constructor).hasId(entity);
    };
    /**
     * Creates a new query builder that can be used to build an sql query.
     */
    EntityManager.prototype.createQueryBuilder = function (entityClass, alias) {
        return this.getRepository(entityClass).createQueryBuilder(alias);
    };
    /**
     * Creates a new entity. If fromRawEntity is given then it creates a new entity and copies all entity properties
     * from this object into a new entity (copies only properties that should be in a new entity).
     */
    EntityManager.prototype.create = function (entityClass, fromRawEntity) {
        return this.getRepository(entityClass).create(fromRawEntity);
    };
    /**
     * Creates a entities from the given array of plain javascript objects.
     */
    EntityManager.prototype.createMany = function (entityClass, copyFromObjects) {
        return this.getRepository(entityClass).createMany(copyFromObjects);
    };
    /**
     * Creates a new entity from the given plan javascript object. If entity already exist in the database, then
     * it loads it (and everything related to it), replaces all values with the new ones from the given object
     * and returns this new entity. This new entity is actually a loaded from the db entity with all properties
     * replaced from the new object.
     */
    EntityManager.prototype.initialize = function (entityClass, object) {
        return this.getRepository(entityClass).initialize(object);
    };
    /**
     * Merges two entities into one new entity.
     */
    EntityManager.prototype.merge = function (entity1, entity2) {
        return this.getRepository(entity1).merge(entity1, entity2);
    };
    /**
     * Persists (saves) a given entity in the database.
     */
    EntityManager.prototype.persist = function (entity) {
        return this.getRepository(entity.constructor).persist(entity);
    };
    /**
     * Removes a given entity from the database.
     */
    EntityManager.prototype.remove = function (entity) {
        return this.getRepository(entity.constructor).remove(entity);
    };
    /**
     * Finds entities that match given conditions.
     */
    EntityManager.prototype.find = function (entityClass, conditionsOrFindOptions, options) {
        if (conditionsOrFindOptions && options) {
            return this.getRepository(entityClass).find(conditionsOrFindOptions, options);
        }
        else if (conditionsOrFindOptions) {
            return this.getRepository(entityClass).find(conditionsOrFindOptions);
        }
        else {
            return this.getRepository(entityClass).find();
        }
    };
    /**
     * Finds entities that match given conditions.
     */
    EntityManager.prototype.findAndCount = function (entityClass, conditionsOrFindOptions, options) {
        if (conditionsOrFindOptions && options) {
            return this.getRepository(entityClass).findAndCount(conditionsOrFindOptions, options);
        }
        else if (conditionsOrFindOptions) {
            return this.getRepository(entityClass).findAndCount(conditionsOrFindOptions);
        }
        else {
            return this.getRepository(entityClass).findAndCount();
        }
    };
    /**
     * Finds first entity that matches given conditions.
     */
    EntityManager.prototype.findOne = function (entityClass, conditionsOrFindOptions, options) {
        if (conditionsOrFindOptions && options) {
            return this.getRepository(entityClass).findOne(conditionsOrFindOptions, options);
        }
        else if (conditionsOrFindOptions) {
            return this.getRepository(entityClass).findOne(conditionsOrFindOptions);
        }
        else {
            return this.getRepository(entityClass).findOne();
        }
    };
    /**
     * Finds entity with given id.
     */
    EntityManager.prototype.findOneById = function (entityClass, id, options) {
        return this.getRepository(entityClass).findOneById(id, options);
    };
    /**
     * Executes raw SQL query and returns raw database results.
     */
    EntityManager.prototype.query = function (query) {
        return this.connection.driver.query(query);
    };
    /**
     * Wraps given function execution (and all operations made there) in a transaction.
     */
    EntityManager.prototype.transaction = function (runInTransaction) {
        var _this = this;
        var runInTransactionResult;
        return this.connection.driver
            .beginTransaction()
            .then(function () { return runInTransaction(); })
            .then(function (result) {
            runInTransactionResult = result;
            return _this.connection.driver.endTransaction();
        })
            .then(function () { return runInTransactionResult; });
    };
    /**
     * Roots are entities that have no ancestors. Finds them all.
     */
    EntityManager.prototype.findRoots = function (entityClass) {
        return this.getTreeRepository(entityClass).findRoots();
    };
    /**
     * Creates a query builder used to get descendants of the entities in a tree.
     */
    EntityManager.prototype.createDescendantsQueryBuilder = function (entityClass, alias, closureTableAlias, entity) {
        return this.getTreeRepository(entityClass).createDescendantsQueryBuilder(alias, closureTableAlias, entity);
    };
    /**
     * Gets all children (descendants) of the given entity. Returns them all in a flat array.
     */
    EntityManager.prototype.findDescendants = function (entityClass, entity) {
        return this.getTreeRepository(entityClass).findDescendants(entity);
    };
    /**
     * Gets all children (descendants) of the given entity. Returns them in a tree - nested into each other.
     */
    EntityManager.prototype.findDescendantsTree = function (entityClass, entity) {
        return this.getTreeRepository(entityClass).findDescendantsTree(entity);
    };
    /**
     * Gets number of descendants of the entity.
     */
    EntityManager.prototype.countDescendants = function (entityClass, entity) {
        return this.getTreeRepository(entityClass).countDescendants(entity);
    };
    /**
     * Creates a query builder used to get ancestors of the entities in the tree.
     */
    EntityManager.prototype.createAncestorsQueryBuilder = function (entityClass, alias, closureTableAlias, entity) {
        return this.getTreeRepository(entityClass).createAncestorsQueryBuilder(alias, closureTableAlias, entity);
    };
    /**
     * Gets all parents (ancestors) of the given entity. Returns them all in a flat array.
     */
    EntityManager.prototype.findAncestors = function (entityClass, entity) {
        return this.getTreeRepository(entityClass).findAncestors(entity);
    };
    /**
     * Gets all parents (ancestors) of the given entity. Returns them in a tree - nested into each other.
     */
    EntityManager.prototype.findAncestorsTree = function (entityClass, entity) {
        return this.getTreeRepository(entityClass).findAncestorsTree(entity);
    };
    /**
     * Gets number of ancestors of the entity.
     */
    EntityManager.prototype.countAncestors = function (entityClass, entity) {
        return this.getTreeRepository(entityClass).countAncestors(entity);
    };
    return EntityManager;
}());
exports.EntityManager = EntityManager;
//# sourceMappingURL=EntityManager.js.map