"use strict";
var EntityMetadata_1 = require("../metadata/EntityMetadata");
var ColumnMetadata_1 = require("../metadata/ColumnMetadata");
var ForeignKeyMetadata_1 = require("../metadata/ForeignKeyMetadata");
var EntityMetadataValidator_1 = require("./EntityMetadataValidator");
var CompositeIndexMetadata_1 = require("../metadata/CompositeIndexMetadata");
var JoinColumnMetadata_1 = require("../metadata/JoinColumnMetadata");
var TableMetadata_1 = require("../metadata/TableMetadata");
var ColumnTypes_1 = require("../metadata/types/ColumnTypes");
var index_1 = require("../index");
/**
 * Aggregates all metadata: table, column, relation into one collection grouped by tables for a given set of classes.
 *
 * @internal
 */
var EntityMetadataBuilder = (function () {
    // -------------------------------------------------------------------------
    // Constructor
    // -------------------------------------------------------------------------
    function EntityMetadataBuilder(namingStrategy) {
        this.namingStrategy = namingStrategy;
        // todo: type in function validation, inverse side function validation
        // todo: check on build for duplicate names, since naming checking was removed from MetadataStorage
        // todo: duplicate name checking for: table, relation, column, index, naming strategy, join tables/columns?
        this.entityValidator = new EntityMetadataValidator_1.EntityMetadataValidator();
    }
    // -------------------------------------------------------------------------
    // Public Methods
    // -------------------------------------------------------------------------
    EntityMetadataBuilder.prototype.mergeIndicesAndCompositeIndices = function (indices, compositeIndices) {
        indices.forEach(function (index) {
            var compositeIndex = new CompositeIndexMetadata_1.CompositeIndexMetadata(index.target, index.name, [index.propertyName]);
            compositeIndex.namingStrategy = index.namingStrategy;
            compositeIndices.add(compositeIndex);
        });
        // later need to check if no duplicate keys in composite indices?
    };
    /**
     * Builds a complete metadata aggregations for the given entity classes.
     */
    EntityMetadataBuilder.prototype.build = function (entityClasses) {
        var _this = this;
        var allMetadataStorage = index_1.defaultMetadataStorage();
        // filter the only metadata we need - those which are bind to the given table classes
        var allTableMetadatas = allMetadataStorage.tableMetadatas.filterByClasses(entityClasses);
        var tableMetadatas = allTableMetadatas.filterByClasses(entityClasses).filter(function (table) { return !table.isAbstract; });
        // set naming strategy
        // allMetadataStorage.tableMetadatas.forEach(tableMetadata => tableMetadata.namingStrategy = this.namingStrategy);
        // allTableMetadatas.forEach(column => column.namingStrategy = this.namingStrategy);
        // entityMetadata.relations.forEach(relation => relation.namingStrategy = this.namingStrategy);
        var entityMetadatas = tableMetadatas.map(function (tableMetadata) {
            var mergedMetadata = allMetadataStorage.mergeWithAbstract(allTableMetadatas, tableMetadata);
            // set naming strategy
            // tableMetadata.namingStrategy = this.namingStrategy;
            mergedMetadata.columnMetadatas.forEach(function (column) { return column.namingStrategy = _this.namingStrategy; });
            mergedMetadata.relationMetadatas.forEach(function (relation) { return relation.namingStrategy = _this.namingStrategy; });
            mergedMetadata.indexMetadatas.forEach(function (relation) { return relation.namingStrategy = _this.namingStrategy; });
            mergedMetadata.compositeIndexMetadatas.forEach(function (relation) { return relation.namingStrategy = _this.namingStrategy; });
            // merge indices and composite indices because simple indices actually are compose indices with only one column
            _this.mergeIndicesAndCompositeIndices(mergedMetadata.indexMetadatas, mergedMetadata.compositeIndexMetadatas);
            // todo: check if multiple tree parent metadatas in validator
            // todo: tree decorators can be used only on closure table (validation)
            // todo: throw error if parent tree metadata was not specified in a closure table
            // create a new entity metadata
            var entityMetadata = new EntityMetadata_1.EntityMetadata(_this.namingStrategy, tableMetadata, mergedMetadata.columnMetadatas, mergedMetadata.relationMetadatas, mergedMetadata.compositeIndexMetadatas);
            // create entity's relations join tables
            entityMetadata.manyToManyRelations.forEach(function (relation) {
                var joinTable = mergedMetadata.joinTableMetadatas.findByProperty(relation.propertyName);
                if (joinTable) {
                    relation.joinTable = joinTable;
                    joinTable.relation = relation;
                }
            });
            // create entity's relations join columns
            entityMetadata.relations.forEach(function (relation) {
                var joinColumn = mergedMetadata.joinColumnMetadatas.findByProperty(relation.propertyName);
                if (joinColumn) {
                    relation.joinColumn = joinColumn;
                    joinColumn.relation = relation;
                }
            });
            // since for many-to-one relations having JoinColumn is not required on decorators level, we need to go
            // throw all of them which don't have JoinColumn decorators and create it for them
            entityMetadata.manyToOneRelations.forEach(function (relation) {
                var joinColumn = mergedMetadata.joinColumnMetadatas.findByProperty(relation.propertyName);
                if (!joinColumn) {
                    joinColumn = new JoinColumnMetadata_1.JoinColumnMetadata(relation.target, relation.propertyName, {});
                    relation.joinColumn = joinColumn;
                    joinColumn.relation = relation;
                }
            });
            return entityMetadata;
        });
        // after all metadatas created we set inverse side (related) entity metadatas for all relation metadatas
        entityMetadatas.forEach(function (entityMetadata) {
            entityMetadata.relations.forEach(function (relation) {
                var inverseEntityMetadata = entityMetadatas.find(function (m) { return m.target === relation.type; });
                if (!inverseEntityMetadata)
                    throw new Error("Entity metadata for " + entityMetadata.name + "#" + relation.propertyName + " was not found.");
                relation.inverseEntityMetadata = inverseEntityMetadata;
            });
        });
        // check for errors in a built metadata schema (we need to check after relationEntityMetadata is set)
        this.entityValidator.validateMany(entityMetadatas);
        // generate columns and foreign keys for tables with relations
        entityMetadatas.forEach(function (metadata) {
            metadata.relationsWithJoinColumns.forEach(function (relation) {
                // find relational column and if it does not exist - add it
                // todo: later add support for propertyInFunction
                var inverseSideColumn = relation.joinColumn.referencedColumn;
                var relationalColumn = metadata.columns.find(function (column) { return column.name === relation.name; }); // todo?: ColumnCollection.findByName
                if (!relationalColumn) {
                    var options = {
                        type: inverseSideColumn.type,
                        oldColumnName: relation.oldColumnName,
                        nullable: relation.isNullable
                    };
                    relationalColumn = new ColumnMetadata_1.ColumnMetadata({
                        target: metadata.target,
                        propertyName: relation.name,
                        propertyType: inverseSideColumn.propertyType,
                        isVirtual: true,
                        options: options
                    });
                    metadata.columns.push(relationalColumn);
                }
                // create and add foreign key
                var foreignKey = new ForeignKeyMetadata_1.ForeignKeyMetadata(metadata.table, [relationalColumn], relation.inverseEntityMetadata.table, [inverseSideColumn], relation.onDelete);
                metadata.foreignKeys.push(foreignKey);
            });
        });
        // generate closure tables
        var closureJunctionEntityMetadatas = [];
        entityMetadatas
            .filter(function (metadata) { return metadata.table.isClosure; })
            .forEach(function (metadata) {
            var closureTableName = _this.namingStrategy.closureJunctionTableName(metadata.table.name);
            var closureJunctionTableMetadata = new TableMetadata_1.TableMetadata(undefined, closureTableName, "closureJunction");
            var columns = [
                new ColumnMetadata_1.ColumnMetadata({
                    propertyType: metadata.primaryColumn.type,
                    options: {
                        length: metadata.primaryColumn.length,
                        type: metadata.primaryColumn.type,
                        name: "ancestor"
                    }
                }),
                new ColumnMetadata_1.ColumnMetadata({
                    propertyType: metadata.primaryColumn.type,
                    options: {
                        length: metadata.primaryColumn.length,
                        type: metadata.primaryColumn.type,
                        name: "descendant"
                    }
                })
            ];
            if (metadata.hasTreeLevelColumn) {
                columns.push(new ColumnMetadata_1.ColumnMetadata({
                    propertyType: ColumnTypes_1.ColumnTypes.INTEGER,
                    options: {
                        type: ColumnTypes_1.ColumnTypes.INTEGER,
                        name: "level"
                    }
                }));
            }
            var closureJunctionEntityMetadata = new EntityMetadata_1.EntityMetadata(_this.namingStrategy, closureJunctionTableMetadata, columns, [], []);
            closureJunctionEntityMetadata.foreignKeys.push(new ForeignKeyMetadata_1.ForeignKeyMetadata(closureJunctionTableMetadata, [columns[0]], metadata.table, [metadata.primaryColumn]), new ForeignKeyMetadata_1.ForeignKeyMetadata(closureJunctionTableMetadata, [columns[1]], metadata.table, [metadata.primaryColumn]));
            closureJunctionEntityMetadatas.push(closureJunctionEntityMetadata);
            metadata.closureJunctionTable = closureJunctionEntityMetadata;
        });
        // generate junction tables with its columns and foreign keys
        var junctionEntityMetadatas = [];
        entityMetadatas.forEach(function (metadata) {
            metadata.ownerManyToManyRelations.map(function (relation) {
                var tableMetadata = new TableMetadata_1.TableMetadata(undefined, relation.joinTable.name, "junction");
                var column1 = relation.joinTable.referencedColumn;
                var column2 = relation.joinTable.inverseReferencedColumn;
                var column1options = {
                    length: column1.length,
                    type: column1.type,
                    name: relation.joinTable.joinColumnName // metadata.table.name + "_" + column1.name
                };
                var column2options = {
                    length: column2.length,
                    type: column2.type,
                    name: relation.joinTable.inverseJoinColumnName // inverseSideMetadata.table.name + "_" + column2.name
                };
                var columns = [
                    new ColumnMetadata_1.ColumnMetadata({
                        propertyType: column2.type,
                        options: column1options
                    }),
                    new ColumnMetadata_1.ColumnMetadata({
                        propertyType: column2.type,
                        options: column2options
                    })
                ];
                var junctionEntityMetadata = new EntityMetadata_1.EntityMetadata(_this.namingStrategy, tableMetadata, columns, [], []);
                junctionEntityMetadata.foreignKeys.push(new ForeignKeyMetadata_1.ForeignKeyMetadata(tableMetadata, [columns[0]], metadata.table, [column1]), new ForeignKeyMetadata_1.ForeignKeyMetadata(tableMetadata, [columns[1]], relation.inverseEntityMetadata.table, [column2]));
                junctionEntityMetadatas.push(junctionEntityMetadata);
                relation.junctionEntityMetadata = junctionEntityMetadata;
                if (relation.hasInverseSide)
                    relation.inverseRelation.junctionEntityMetadata = junctionEntityMetadata;
            });
        });
        return entityMetadatas
            .concat(junctionEntityMetadatas)
            .concat(closureJunctionEntityMetadatas);
    };
    return EntityMetadataBuilder;
}());
exports.EntityMetadataBuilder = EntityMetadataBuilder;
//# sourceMappingURL=EntityMetadataBuilder.js.map