"use strict";
var UsingJoinTableIsNotAllowedError_1 = require("./error/UsingJoinTableIsNotAllowedError");
var UsingJoinTableOnlyOnOneSideAllowedError_1 = require("./error/UsingJoinTableOnlyOnOneSideAllowedError");
var UsingJoinColumnIsNotAllowedError_1 = require("./error/UsingJoinColumnIsNotAllowedError");
var UsingJoinColumnOnlyOnOneSideAllowedError_1 = require("./error/UsingJoinColumnOnlyOnOneSideAllowedError");
var MissingJoinColumnError_1 = require("./error/MissingJoinColumnError");
var MissingJoinTableError_1 = require("./error/MissingJoinTableError");
var MissingPrimaryColumnError_1 = require("./error/MissingPrimaryColumnError");
/**
 * Validates built entity metadatas.
 *
 * @internal
 */
var EntityMetadataValidator = (function () {
    function EntityMetadataValidator() {
    }
    // -------------------------------------------------------------------------
    // Public Methods
    // -------------------------------------------------------------------------
    /**
     * Validates all given entity metadatas.
     */
    EntityMetadataValidator.prototype.validateMany = function (entityMetadatas) {
        var _this = this;
        entityMetadatas.forEach(function (entityMetadata) { return _this.validate(entityMetadata); });
    };
    /**
     * Validates given entity metadata.
     */
    EntityMetadataValidator.prototype.validate = function (entityMetadata) {
        // check if table metadata has an id
        if (!entityMetadata.primaryColumn)
            throw new MissingPrimaryColumnError_1.MissingPrimaryColumnError(entityMetadata);
        entityMetadata.relations.forEach(function (relation) {
            // check join tables:
            // using JoinTable is possible only on one side of the many-to-many relation
            if (relation.joinTable) {
                if (!relation.isManyToMany)
                    throw new UsingJoinTableIsNotAllowedError_1.UsingJoinTableIsNotAllowedError(entityMetadata, relation);
                // if there is inverse side of the relation, then check if it does not have join table too
                if (relation.hasInverseSide && relation.inverseRelation.joinTable)
                    throw new UsingJoinTableOnlyOnOneSideAllowedError_1.UsingJoinTableOnlyOnOneSideAllowedError(entityMetadata, relation);
            }
            // check join columns:
            // using JoinColumn is possible only on one side of the relation and on one-to-one, many-to-one relation types
            // first check if relation is one-to-one or many-to-one
            if (relation.joinColumn) {
                // join column can be applied only on one-to-one and many-to-one relations
                if (!relation.isOneToOne && !relation.isManyToOne)
                    throw new UsingJoinColumnIsNotAllowedError_1.UsingJoinColumnIsNotAllowedError(entityMetadata, relation);
                // if there is inverse side of the relation, then check if it does not have join table too
                if (relation.hasInverseSide && relation.inverseRelation.joinColumn && relation.isOneToOne)
                    throw new UsingJoinColumnOnlyOnOneSideAllowedError_1.UsingJoinColumnOnlyOnOneSideAllowedError(entityMetadata, relation);
            }
            // if its a one-to-one relation and JoinColumn is missing on both sides of the relation
            // or its one-side relation without JoinColumn we should give an error
            if (!relation.joinColumn && relation.isOneToOne && (!relation.hasInverseSide || !relation.inverseRelation.joinColumn))
                throw new MissingJoinColumnError_1.MissingJoinColumnError(entityMetadata, relation);
            // if its a many-to-many relation and JoinTable is missing on both sides of the relation
            // or its one-side relation without JoinTable we should give an error
            if (!relation.joinTable && relation.isManyToMany && (!relation.hasInverseSide || !relation.inverseRelation.joinTable))
                throw new MissingJoinTableError_1.MissingJoinTableError(entityMetadata, relation);
            // todo: validate if its one-to-one and side which does not have join column MUST have inverse side
            // todo: validate if its many-to-many and side which does not have join table MUST have inverse side
            // todo: if there is a relation, and inverse side is specified only on one side, shall we give error
            // todo: with message like: "Inverse side is specified only on one side of the relationship. Specify on other side too to prevent confusion".
        });
    };
    return EntityMetadataValidator;
}());
exports.EntityMetadataValidator = EntityMetadataValidator;
//# sourceMappingURL=EntityMetadataValidator.js.map