"use strict";
var RelationTypes_1 = require("./types/RelationTypes");
/**
 * Contains all entity metadata.
 */
var EntityMetadata = (function () {
    // -------------------------------------------------------------------------
    // Constructor
    // -------------------------------------------------------------------------
    function EntityMetadata(namingStrategy, tableMetadata, columnMetadatas, relationMetadatas, compositeIndexMetadatas) {
        var _this = this;
        this.foreignKeys = [];
        this.namingStrategy = namingStrategy;
        this.table = tableMetadata;
        this.columns = columnMetadatas;
        this.relations = relationMetadatas;
        this.compositeIndices = compositeIndexMetadatas;
        this.table.entityMetadata = this;
        this.relations.forEach(function (relation) { return relation.entityMetadata = _this; });
        this.compositeIndices.forEach(function (index) { return index.entityMetadata = _this; });
    }
    Object.defineProperty(EntityMetadata.prototype, "name", {
        // -------------------------------------------------------------------------
        // Accessors
        // -------------------------------------------------------------------------
        get: function () {
            if (!this.table) {
                throw new Error("No table target set to the entity metadata.");
            }
            if (this.table.target)
                return this.table.target.name;
            return this.table.name;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(EntityMetadata.prototype, "target", {
        get: function () {
            return this.table.target;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(EntityMetadata.prototype, "oneToOneRelations", {
        get: function () {
            return this.relations.filter(function (relation) { return relation.relationType === RelationTypes_1.RelationTypes.ONE_TO_ONE; });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(EntityMetadata.prototype, "ownerOneToOneRelations", {
        get: function () {
            return this.relations.filter(function (relation) { return relation.relationType === RelationTypes_1.RelationTypes.ONE_TO_ONE && relation.isOwning; });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(EntityMetadata.prototype, "oneToManyRelations", {
        get: function () {
            return this.relations.filter(function (relation) { return relation.relationType === RelationTypes_1.RelationTypes.ONE_TO_MANY; });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(EntityMetadata.prototype, "manyToOneRelations", {
        get: function () {
            return this.relations.filter(function (relation) { return relation.relationType === RelationTypes_1.RelationTypes.MANY_TO_ONE; });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(EntityMetadata.prototype, "manyToManyRelations", {
        get: function () {
            return this.relations.filter(function (relation) { return relation.relationType === RelationTypes_1.RelationTypes.MANY_TO_MANY; });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(EntityMetadata.prototype, "ownerManyToManyRelations", {
        get: function () {
            return this.relations.filter(function (relation) { return relation.relationType === RelationTypes_1.RelationTypes.MANY_TO_MANY && relation.isOwning; });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(EntityMetadata.prototype, "relationsWithJoinColumns", {
        get: function () {
            return this.ownerOneToOneRelations.concat(this.manyToOneRelations);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(EntityMetadata.prototype, "hasPrimaryColumn", {
        get: function () {
            return !!this.columns.find(function (column) { return column.isPrimary; });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(EntityMetadata.prototype, "primaryColumn", {
        get: function () {
            var primaryKey = this.columns.find(function (column) { return column.isPrimary; });
            if (!primaryKey)
                throw new Error("Primary key is not set for the " + this.name + " entity.");
            return primaryKey;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(EntityMetadata.prototype, "hasCreateDateColumn", {
        get: function () {
            return !!this.columns.find(function (column) { return column.mode === "createDate"; });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(EntityMetadata.prototype, "createDateColumn", {
        get: function () {
            var column = this.columns.find(function (column) { return column.mode === "createDate"; });
            if (!column)
                throw new Error("CreateDateColumn was not found in entity " + this.name);
            return column;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(EntityMetadata.prototype, "hasUpdateDateColumn", {
        get: function () {
            return !!this.columns.find(function (column) { return column.mode === "updateDate"; });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(EntityMetadata.prototype, "updateDateColumn", {
        get: function () {
            var column = this.columns.find(function (column) { return column.mode === "updateDate"; });
            if (!column)
                throw new Error("UpdateDateColumn was not found in entity " + this.name);
            return column;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(EntityMetadata.prototype, "hasVersionColumn", {
        get: function () {
            return !!this.columns.find(function (column) { return column.mode === "version"; });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(EntityMetadata.prototype, "versionColumn", {
        get: function () {
            var column = this.columns.find(function (column) { return column.mode === "version"; });
            if (!column)
                throw new Error("VersionColumn was not found in entity " + this.name);
            return column;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(EntityMetadata.prototype, "hasTreeChildrenCountColumn", {
        get: function () {
            return !!this.columns.find(function (column) { return column.mode === "treeChildrenCount"; });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(EntityMetadata.prototype, "treeChildrenCountColumn", {
        get: function () {
            var column = this.columns.find(function (column) { return column.mode === "treeChildrenCount"; });
            if (!column)
                throw new Error("TreeChildrenCountColumn was not found in entity " + this.name);
            return column;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(EntityMetadata.prototype, "hasTreeLevelColumn", {
        get: function () {
            return !!this.columns.find(function (column) { return column.mode === "treeLevel"; });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(EntityMetadata.prototype, "treeLevelColumn", {
        get: function () {
            var column = this.columns.find(function (column) { return column.mode === "treeLevel"; });
            if (!column)
                throw new Error("TreeLevelColumn was not found in entity " + this.name);
            return column;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(EntityMetadata.prototype, "hasPrimaryKey", {
        get: function () {
            return !!this.primaryColumn;
        },
        enumerable: true,
        configurable: true
    });
    // -------------------------------------------------------------------------
    // Public Methods
    // -------------------------------------------------------------------------
    /**
     * Creates a new entity.
     */
    EntityMetadata.prototype.create = function () {
        return new this.table.target();
    };
    EntityMetadata.prototype.createPropertiesMap = function () {
        var entity = {};
        this.columns.forEach(function (column) { return entity[column.propertyName] = column.propertyName; });
        this.relations.forEach(function (relation) { return entity[relation.propertyName] = relation.propertyName; });
        return entity;
    };
    EntityMetadata.prototype.getEntityId = function (entity) {
        return entity[this.primaryColumn.propertyName];
    };
    EntityMetadata.prototype.hasColumnWithPropertyName = function (propertyName) {
        return !!this.columns.find(function (column) { return column.propertyName === propertyName; });
    };
    EntityMetadata.prototype.hasColumnWithDbName = function (name) {
        return !!this.columns.find(function (column) { return column.name === name; });
    };
    EntityMetadata.prototype.hasRelationWithPropertyName = function (propertyName) {
        return !!this.relations.find(function (relation) { return relation.propertyName === propertyName; });
    };
    EntityMetadata.prototype.findRelationWithPropertyName = function (propertyName) {
        var relation = this.relations.find(function (relation) { return relation.propertyName === propertyName; });
        if (!relation)
            throw new Error("Relation with property name " + propertyName + " in " + this.name + " entity was not found.");
        return relation;
    };
    EntityMetadata.prototype.hasRelationWithDbName = function (dbName) {
        return !!this.relations.find(function (relation) { return relation.name === dbName; });
    };
    EntityMetadata.prototype.findRelationWithDbName = function (propertyName) {
        var relation = this.relations.find(function (relation) { return relation.name === propertyName; });
        if (!relation)
            throw new Error("Relation with name " + propertyName + " in " + this.name + " entity was not found.");
        return relation;
    };
    EntityMetadata.prototype.hasRelationWithOneWithPropertyName = function (propertyName) {
        return !!this.relations.find(function (relation) { return relation.propertyName === propertyName && (relation.isOneToMany || relation.isOneToOne); });
    };
    EntityMetadata.prototype.findRelationWithOneWithPropertyName = function (propertyName) {
        var relation = this.relations.find(function (relation) { return relation.propertyName === propertyName && (relation.isOneToMany || relation.isOneToOne); });
        if (!relation)
            throw new Error("Relation with one with property name " + propertyName + " in " + this.name + " entity was not found.");
        return relation;
    };
    EntityMetadata.prototype.hasRelationWithOneWithDbName = function (name) {
        return !!this.relations.find(function (relation) { return relation.name === name && (relation.isOneToMany || relation.isOneToOne); });
    };
    EntityMetadata.prototype.findRelationWithOneWithDbName = function (name) {
        var relation = this.relations.find(function (relation) { return relation.name === name && (relation.isOneToMany || relation.isOneToOne); });
        if (!relation)
            throw new Error("Relation with one with name " + name + " in " + this.name + " entity was not found.");
        return relation;
    };
    EntityMetadata.prototype.hasRelationWithManyWithPropertyName = function (name) {
        return !!this.relations.find(function (relation) { return relation.propertyName === name && (relation.isManyToOne || relation.isManyToMany); });
    };
    EntityMetadata.prototype.findRelationWithManyWithPropertyName = function (name) {
        var relation = this.relations.find(function (relation) { return relation.propertyName === name && (relation.isManyToOne || relation.isManyToMany); });
        if (!relation)
            throw new Error("Relation with many with property name " + name + " in " + this.name + " entity was not found.");
        return relation;
    };
    EntityMetadata.prototype.hasRelationWithManyWithDbName = function (name) {
        return !!this.relations.find(function (relation) { return relation.name === name && (relation.isManyToOne || relation.isManyToMany); });
    };
    EntityMetadata.prototype.findRelationWithManyWithDbName = function (name) {
        var relation = this.relations.find(function (relation) { return relation.name === name && (relation.isManyToOne || relation.isManyToMany); });
        if (!relation)
            throw new Error("Relation with many with name " + name + " in " + this.name + " entity was not found.");
        return relation;
    };
    Object.defineProperty(EntityMetadata.prototype, "hasTreeParentRelation", {
        get: function () {
            return !!this.relations.find(function (relation) { return relation.isTreeParent; });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(EntityMetadata.prototype, "treeParentRelation", {
        get: function () {
            var relation = this.relations.find(function (relation) { return relation.isTreeParent; });
            if (!relation)
                throw new Error("TreeParent relation was not found in entity " + this.name);
            return relation;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(EntityMetadata.prototype, "hasTreeChildrenRelation", {
        get: function () {
            return !!this.relations.find(function (relation) { return relation.isTreeChildren; });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(EntityMetadata.prototype, "treeChildrenRelation", {
        get: function () {
            var relation = this.relations.find(function (relation) { return relation.isTreeChildren; });
            if (!relation)
                throw new Error("TreeParent relation was not found in entity " + this.name);
            return relation;
        },
        enumerable: true,
        configurable: true
    });
    return EntityMetadata;
}());
exports.EntityMetadata = EntityMetadata;
//# sourceMappingURL=EntityMetadata.js.map