import { PropertyMetadata } from "./PropertyMetadata";
import { RelationType } from "./types/RelationTypes";
import { NamingStrategyInterface } from "../naming-strategy/NamingStrategyInterface";
import { EntityMetadata } from "./EntityMetadata";
import { OnDeleteType } from "./ForeignKeyMetadata";
import { JoinTableMetadata } from "./JoinTableMetadata";
import { JoinColumnMetadata } from "./JoinColumnMetadata";
import { RelationMetadataArgs } from "./args/RelationMetadataArgs";
/**
 * Function that returns a type of the field. Returned value must be a class used on the relation.
 */
export declare type RelationTypeInFunction = ((type?: any) => Function);
/**
 * Contains the name of the property of the object, or the function that returns this name.
 */
export declare type PropertyTypeInFunction<T> = string | ((t: T) => string | any);
/**
 * This metadata interface contains all information about some document's relation.
 */
export declare class RelationMetadata extends PropertyMetadata {
    /**
     * Naming strategy used to generate and normalize column name.
     */
    namingStrategy: NamingStrategyInterface;
    /**
     * Its own entity metadata.
     */
    entityMetadata: EntityMetadata;
    /**
     * Related entity metadata.
     */
    inverseEntityMetadata: EntityMetadata;
    /**
     * Junction entity metadata.
     */
    junctionEntityMetadata: EntityMetadata;
    /**
     * Join table metadata.
     */
    joinTable: JoinTableMetadata;
    /**
     * Join column metadata.
     */
    joinColumn: JoinColumnMetadata;
    /**
     * Indicates if this is a parent (can be only many-to-one relation) relation in the tree tables.
     */
    readonly isTreeParent: boolean;
    /**
     * Indicates if this is a children (can be only one-to-many relation) relation in the tree tables.
     */
    readonly isTreeChildren: boolean;
    /**
     * Relation type.
     */
    readonly relationType: RelationType;
    /**
     * If set to true then it means that related object can be allowed to be inserted to the db.
     */
    readonly isCascadeInsert: boolean;
    /**
     * If set to true then it means that related object can be allowed to be updated in the db.
     */
    readonly isCascadeUpdate: boolean;
    /**
     * If set to true then it means that related object can be allowed to be remove from the db.
     */
    readonly isCascadeRemove: boolean;
    /**
     * Indicates if relation column value can be nullable or not.
     */
    readonly isNullable: boolean;
    /**
     * Old column name.
     */
    readonly oldColumnName: string;
    /**
     * What to do with a relation on deletion of the row containing a foreign key.
     */
    readonly onDelete: OnDeleteType;
    /**
     * The real reflected property type.
     */
    readonly propertyType: any;
    /**
     * Column name for this relation.
     */
    private _name;
    /**
     * The type of the field.
     */
    private _type;
    /**
     * Inverse side of the relation.
     */
    private _inverseSideProperty;
    constructor(args: RelationMetadataArgs);
    readonly name: string;
    readonly referencedColumnName: string;
    /**
     * Indicates if this side is an owner of this relation.
     */
    readonly isOwning: boolean | JoinTableMetadata | JoinColumnMetadata;
    readonly type: Function;
    readonly inverseSideProperty: string;
    readonly inverseRelation: RelationMetadata;
    readonly isOneToOne: boolean;
    readonly isOneToMany: boolean;
    readonly isManyToOne: boolean;
    readonly isManyToMany: boolean;
    readonly hasInverseSide: boolean;
    readonly isLazy: boolean;
    private computeInverseSide(inverseSide);
}
