"use strict";
var PersistOperation_1 = require("./operation/PersistOperation");
var InsertOperation_1 = require("./operation/InsertOperation");
var UpdateByRelationOperation_1 = require("./operation/UpdateByRelationOperation");
var UpdateOperation_1 = require("./operation/UpdateOperation");
var CascadesNotAllowedError_1 = require("./error/CascadesNotAllowedError");
var RemoveOperation_1 = require("./operation/RemoveOperation");
var UpdateByInverseSideOperation_1 = require("./operation/UpdateByInverseSideOperation");
/**
 * 1. collect all exist objects from the db entity
 * 2. collect all objects from the new entity
 * 3. first need to go throw all relations of the new entity and:
 *      3.1. find all objects that are new (e.g. cascade="insert") by comparing ids from the exist objects
 *      3.2. check if relation has rights to do cascade operation and throw exception if it cannot
 *      3.3. save new objects for insert operation
 * 4. second need to go throw all relations of the db entity and:
 *      4.1. find all objects that are removed (e.g. cascade="remove") by comparing data with collected objects of the new entity
 *      4.2. check if relation has rights to do cascade operation and throw exception if it cannot
 *      4.3. save new objects for remove operation
 * 5. third need to go throw collection of all new entities
 *      5.1. compare with entities from the collection of db entities, find difference and generate a change set
 *      5.2. check if relation has rights to do cascade operation and throw exception if it cannot
 *      5.3.
 * 6. go throw all relations and find junction
 *      6.1.
 *
 * if relation has "all" then all of above:
 * if relation has "insert" it can insert a new entity
 * if relation has "update" it can only update related entity
 * if relation has "remove" it can only remove related entity
 * @internal
 */
var EntityPersistOperationBuilder = (function () {
    // -------------------------------------------------------------------------
    // Constructor
    // -------------------------------------------------------------------------
    function EntityPersistOperationBuilder(entityMetadatas) {
        this.entityMetadatas = entityMetadatas;
        // -------------------------------------------------------------------------
        // Properties
        // -------------------------------------------------------------------------
        this.strictCascadesMode = false;
    }
    // -------------------------------------------------------------------------
    // Public Methods
    // -------------------------------------------------------------------------
    /**
     * Finds columns and relations from entity2 which does not exist or does not match in entity1.
     */
    EntityPersistOperationBuilder.prototype.buildFullPersistment = function (metadata, dbEntity, persistedEntity, dbEntities, allPersistedEntities) {
        // const dbEntities = this.extractObjectsById(dbEntity, metadata);
        // const allPersistedEntities = this.extractObjectsById(persistedEntity, metadata);
        var persistOperation = new PersistOperation_1.PersistOperation();
        persistOperation.dbEntity = dbEntity;
        persistOperation.persistedEntity = persistedEntity;
        persistOperation.allDbEntities = dbEntities;
        persistOperation.allPersistedEntities = allPersistedEntities;
        persistOperation.inserts = this.findCascadeInsertedEntities(persistedEntity, dbEntities);
        persistOperation.updatesByRelations = this.updateRelations(persistOperation.inserts, persistedEntity);
        persistOperation.updatesByInverseRelations = this.updateInverseRelations(metadata, dbEntity, persistedEntity);
        persistOperation.updates = this.findCascadeUpdateEntities(persistOperation.updatesByRelations, metadata, dbEntity, persistedEntity, dbEntities);
        persistOperation.junctionInserts = this.findJunctionInsertOperations(metadata, persistedEntity, dbEntities);
        persistOperation.removes = this.findCascadeRemovedEntities(metadata, dbEntity, allPersistedEntities, undefined, undefined, undefined);
        persistOperation.junctionRemoves = this.findJunctionRemoveOperations(metadata, dbEntity, allPersistedEntities);
        // persistOperation.log();
        return persistOperation;
    };
    /**
     * Finds columns and relations from entity2 which does not exist or does not match in entity1.
     */
    EntityPersistOperationBuilder.prototype.buildOnlyRemovement = function (metadata, dbEntity, persistedEntity, dbEntities, allPersistedEntities) {
        // const dbEntities = this.extractObjectsById(dbEntity, metadata);
        // const allEntities = this.extractObjectsById(newEntity, metadata);
        var persistOperation = new PersistOperation_1.PersistOperation();
        persistOperation.dbEntity = dbEntity;
        persistOperation.persistedEntity = persistedEntity;
        persistOperation.allDbEntities = dbEntities;
        persistOperation.allPersistedEntities = allPersistedEntities;
        persistOperation.removes = this.findCascadeRemovedEntities(metadata, dbEntity, allPersistedEntities, undefined, undefined, undefined);
        persistOperation.junctionRemoves = this.findJunctionRemoveOperations(metadata, dbEntity, allPersistedEntities);
        return persistOperation;
    };
    // -------------------------------------------------------------------------
    // Private Methods
    // -------------------------------------------------------------------------
    EntityPersistOperationBuilder.prototype.findCascadeInsertedEntities = function (newEntity, dbEntities, fromRelation, operations) {
        var _this = this;
        if (operations === void 0) { operations = []; }
        // const metadata = this.connection.getEntityMetadata(newEntity.constructor);
        var metadata = this.entityMetadatas.findByTarget(newEntity.constructor);
        var isObjectNew = !this.findEntityWithId(dbEntities, metadata.target, newEntity[metadata.primaryColumn.propertyName]);
        // if object is new and should be inserted, we check if cascades are allowed before add it to operations list
        if (isObjectNew && fromRelation && !this.checkCascadesAllowed("insert", metadata, fromRelation)) {
            return operations; // looks like object is new here, but cascades are not allowed - then we should stop iteration
        }
        else if (isObjectNew && !operations.find(function (o) { return o.entity === newEntity; })) {
            operations.push(new InsertOperation_1.InsertOperation(newEntity));
        }
        metadata.relations.forEach(function (relation) {
            var value = _this.getEntityRelationValue(relation, newEntity);
            if (!value)
                return;
            if (value instanceof Array) {
                value.map(function (subValue) { return _this.findCascadeInsertedEntities(subValue, dbEntities, relation, operations); });
            }
            else {
                _this.findCascadeInsertedEntities(value, dbEntities, relation, operations);
            }
        });
        return operations;
    };
    EntityPersistOperationBuilder.prototype.findCascadeUpdateEntities = function (updatesByRelations, metadata, dbEntity, newEntity, dbEntities, fromRelation, operations) {
        var _this = this;
        if (operations === void 0) { operations = []; }
        if (!dbEntity)
            return operations;
        var diffColumns = this.diffColumns(metadata, newEntity, dbEntity);
        var diffRelations = this.diffRelations(updatesByRelations, metadata, newEntity, dbEntity);
        if (diffColumns.length && fromRelation && !this.checkCascadesAllowed("update", metadata, fromRelation)) {
            return operations;
        }
        else if (diffColumns.length || diffRelations.length) {
            var entityId = newEntity[metadata.primaryColumn.propertyName];
            if (entityId)
                operations.push(new UpdateOperation_1.UpdateOperation(newEntity, entityId, diffColumns, diffRelations));
        }
        metadata.relations.forEach(function (relation) {
            var relMetadata = relation.inverseEntityMetadata;
            var relationIdColumnName = relMetadata.primaryColumn.propertyName;
            var value = _this.getEntityRelationValue(relation, newEntity);
            var referencedColumnName = relation.isOwning ? relation.referencedColumnName : relation.inverseRelation.referencedColumnName;
            // const dbValue = this.getEntityRelationValue(relation, dbEntity);
            if (!value /* || !dbValue*/)
                return;
            if (value instanceof Array) {
                value.forEach(function (subEntity) {
                    /*const subDbEntity = dbValue.find((subDbEntity: any) => {
                        return subDbEntity[relationIdColumnName] === subEntity[relationIdColumnName];
                    });*/
                    var dbValue = dbEntities.find(function (dbValue) {
                        return dbValue.entity.constructor === subEntity.constructor && dbValue.entity[referencedColumnName] === subEntity[relationIdColumnName];
                    });
                    if (dbValue)
                        _this.findCascadeUpdateEntities(updatesByRelations, relMetadata, dbValue.entity, subEntity, dbEntities, relation, operations);
                });
            }
            else {
                var dbValue = dbEntities.find(function (dbValue) {
                    return dbValue.entity.constructor === value.constructor && dbValue.entity[referencedColumnName] === value[relationIdColumnName];
                });
                if (dbValue)
                    _this.findCascadeUpdateEntities(updatesByRelations, relMetadata, dbValue.entity, value, dbEntities, relation, operations);
            }
        });
        return operations;
    };
    EntityPersistOperationBuilder.prototype.findCascadeRemovedEntities = function (metadata, dbEntity, allPersistedEntities, fromRelation, fromMetadata, fromEntityId, parentAlreadyRemoved) {
        var _this = this;
        if (parentAlreadyRemoved === void 0) { parentAlreadyRemoved = false; }
        var operations = [];
        if (!dbEntity)
            return operations;
        var entityId = dbEntity[metadata.primaryColumn.propertyName];
        var isObjectRemoved = parentAlreadyRemoved || !this.findEntityWithId(allPersistedEntities, metadata.target, entityId);
        // if object is removed and should be removed, we check if cascades are allowed before add it to operations list
        if (isObjectRemoved && fromRelation && !this.checkCascadesAllowed("remove", metadata, fromRelation)) {
            return operations; // looks like object is removed here, but cascades are not allowed - then we should stop iteration
        }
        else if (isObjectRemoved) {
            operations.push(new RemoveOperation_1.RemoveOperation(dbEntity, entityId, fromMetadata, fromRelation, fromEntityId));
        }
        metadata.relations.forEach(function (relation) {
            var dbValue = _this.getEntityRelationValue(relation, dbEntity);
            var relMetadata = relation.inverseEntityMetadata;
            if (!dbValue)
                return;
            if (dbValue instanceof Array) {
                dbValue.forEach(function (subDbEntity) {
                    var relationOperations = _this.findCascadeRemovedEntities(relMetadata, subDbEntity, allPersistedEntities, relation, metadata, dbEntity[metadata.primaryColumn.propertyName], isObjectRemoved);
                    relationOperations.forEach(function (o) { return operations.push(o); });
                });
            }
            else {
                var relationOperations = _this.findCascadeRemovedEntities(relMetadata, dbValue, allPersistedEntities, relation, metadata, dbEntity[metadata.primaryColumn.propertyName], isObjectRemoved);
                relationOperations.forEach(function (o) { return operations.push(o); });
            }
        }, []);
        return operations;
    };
    EntityPersistOperationBuilder.prototype.updateInverseRelations = function (metadata, dbEntity, newEntity, operations) {
        if (operations === void 0) { operations = []; }
        metadata.relations
            .filter(function (relation) { return relation.isOneToMany; }) // todo: maybe need to check isOneToOne and not owner
            .forEach(function (relation) {
            // to find new objects in relation go throw all objects in newEntity and check if they don't exist in dbEntity
            if (newEntity && newEntity[relation.propertyName] instanceof Array) {
                newEntity[relation.propertyName].filter(function (newSubEntity) {
                    if (!dbEntity /* are you sure about this? */ || !dbEntity[relation.propertyName])
                        return true;
                    return !dbEntity[relation.propertyName].find(function (dbSubEntity) {
                        return relation.inverseEntityMetadata.getEntityId(newSubEntity) === relation.inverseEntityMetadata.getEntityId(dbSubEntity);
                    });
                }).forEach(function (subEntity) {
                    operations.push(new UpdateByInverseSideOperation_1.UpdateByInverseSideOperation("update", subEntity, newEntity, relation));
                });
            }
            // we also need to find removed elements. to find them need to traverse dbEntity and find its elements missing in newEntity
            if (dbEntity && dbEntity[relation.propertyName] instanceof Array) {
                dbEntity[relation.propertyName].filter(function (dbSubEntity) {
                    if (!newEntity /* are you sure about this? */ || !newEntity[relation.propertyName])
                        return true;
                    return !newEntity[relation.propertyName].find(function (newSubEntity) {
                        return relation.inverseEntityMetadata.getEntityId(dbSubEntity) === relation.inverseEntityMetadata.getEntityId(newSubEntity);
                    });
                }).forEach(function (subEntity) {
                    operations.push(new UpdateByInverseSideOperation_1.UpdateByInverseSideOperation("remove", subEntity, newEntity, relation));
                });
            }
        });
        return operations;
    };
    /**
     * To update relation, you need:
     *   update table where this relation (owner side)
     *   set its relation property to inserted id
     *   where
     *
     */
    EntityPersistOperationBuilder.prototype.updateRelations = function (insertOperations, newEntity) {
        var _this = this;
        return insertOperations.reduce(function (operations, insertOperation) {
            return operations.concat(_this.findRelationsWithEntityInside(insertOperation, newEntity));
        }, []);
    };
    EntityPersistOperationBuilder.prototype.findRelationsWithEntityInside = function (insertOperation, entityToSearchIn) {
        var _this = this;
        // updateByt metadata = this.connection.getEntityMetadata(entityToSearchIn.constructor);
        var metadata = this.entityMetadatas.findByTarget(entityToSearchIn.constructor);
        var operations = [];
        metadata.relations.forEach(function (relation) {
            var value = _this.getEntityRelationValue(relation, entityToSearchIn);
            if (!value)
                return;
            if (value instanceof Array) {
                value.forEach(function (sub) {
                    if (!relation.isManyToMany && sub === insertOperation.entity)
                        operations.push(new UpdateByRelationOperation_1.UpdateByRelationOperation(entityToSearchIn, insertOperation, relation));
                    var subOperations = _this.findRelationsWithEntityInside(insertOperation, sub);
                    subOperations.forEach(function (o) { return operations.push(o); });
                });
            }
            else if (value) {
                if (value === insertOperation.entity) {
                    operations.push(new UpdateByRelationOperation_1.UpdateByRelationOperation(entityToSearchIn, insertOperation, relation));
                }
                var subOperations = _this.findRelationsWithEntityInside(insertOperation, value);
                subOperations.forEach(function (o) { return operations.push(o); });
            }
        });
        return operations;
    };
    EntityPersistOperationBuilder.prototype.findJunctionInsertOperations = function (metadata, newEntity, dbEntities, isRoot) {
        var _this = this;
        if (isRoot === void 0) { isRoot = true; }
        var dbEntity = dbEntities.find(function (dbEntity) {
            return dbEntity.id === newEntity[metadata.primaryColumn.propertyName] && dbEntity.entity.constructor === metadata.target;
        });
        return metadata.relations
            .filter(function (relation) { return newEntity[relation.propertyName] !== null && newEntity[relation.propertyName] !== undefined; })
            .reduce(function (operations, relation) {
            var relationMetadata = relation.inverseEntityMetadata;
            var relationIdProperty = relationMetadata.primaryColumn.propertyName;
            var value = _this.getEntityRelationValue(relation, newEntity);
            var dbValue = dbEntity ? _this.getEntityRelationValue(relation, dbEntity.entity) : null;
            if (value instanceof Array) {
                value.forEach(function (subEntity) {
                    if (relation.isManyToMany) {
                        var has = !dbValue || !dbValue.find(function (e) { return e[relationIdProperty] === subEntity[relationIdProperty]; });
                        if (has) {
                            operations.push({
                                metadata: relation.junctionEntityMetadata,
                                entity1: newEntity,
                                entity2: subEntity
                            });
                        }
                    }
                    if (isRoot || _this.checkCascadesAllowed("update", metadata, relation)) {
                        var subOperations = _this.findJunctionInsertOperations(relationMetadata, subEntity, dbEntities, false);
                        subOperations.forEach(function (o) { return operations.push(o); });
                    }
                });
            }
            else {
                if (isRoot || _this.checkCascadesAllowed("update", metadata, relation)) {
                    var subOperations = _this.findJunctionInsertOperations(relationMetadata, value, dbEntities, false);
                    subOperations.forEach(function (o) { return operations.push(o); });
                }
            }
            return operations;
        }, []);
    };
    EntityPersistOperationBuilder.prototype.findJunctionRemoveOperations = function (metadata, dbEntity, newEntities, isRoot) {
        var _this = this;
        if (isRoot === void 0) { isRoot = true; }
        if (!dbEntity)
            return [];
        var newEntity = newEntities.find(function (newEntity) {
            return newEntity.id === dbEntity[metadata.primaryColumn.propertyName] && newEntity.entity.constructor === metadata.target;
        });
        return metadata.relations
            .filter(function (relation) { return dbEntity[relation.propertyName] !== null && dbEntity[relation.propertyName] !== undefined; })
            .reduce(function (operations, relation) {
            var relationMetadata = relation.inverseEntityMetadata;
            var relationIdProperty = relationMetadata.primaryColumn.propertyName;
            var value = newEntity ? _this.getEntityRelationValue(relation, newEntity.entity) : null;
            var dbValue = _this.getEntityRelationValue(relation, dbEntity);
            if (dbValue instanceof Array) {
                dbValue.forEach(function (subEntity) {
                    if (relation.isManyToMany) {
                        var has = !value || !value.find(function (e) { return e[relationIdProperty] === subEntity[relationIdProperty]; });
                        if (has) {
                            operations.push({
                                metadata: relation.junctionEntityMetadata,
                                entity1: dbEntity,
                                entity2: subEntity
                            });
                        }
                    }
                    if (isRoot || _this.checkCascadesAllowed("update", metadata, relation)) {
                        var subOperations = _this.findJunctionRemoveOperations(relationMetadata, subEntity, newEntities, false);
                        subOperations.forEach(function (o) { return operations.push(o); });
                    }
                });
            }
            else {
                if (isRoot || _this.checkCascadesAllowed("update", metadata, relation)) {
                    var subOperations = _this.findJunctionRemoveOperations(relationMetadata, dbValue, newEntities, false);
                    subOperations.forEach(function (o) { return operations.push(o); });
                }
            }
            return operations;
        }, []);
    };
    EntityPersistOperationBuilder.prototype.diffColumns = function (metadata, newEntity, dbEntity) {
        return metadata.columns
            .filter(function (column) { return !column.isVirtual && !column.isUpdateDate && !column.isVersion && !column.isCreateDate; })
            .filter(function (column) { return newEntity[column.propertyName] !== dbEntity[column.propertyName]; })
            .filter(function (column) {
            // filter out "relational columns" only in the case if there is a relation object in entity
            if (metadata.hasRelationWithDbName(column.propertyName)) {
                var relation = metadata.findRelationWithDbName(column.propertyName);
                if (newEntity[relation.propertyName] !== null && newEntity[relation.propertyName] !== undefined)
                    return false;
            }
            return true;
        });
    };
    EntityPersistOperationBuilder.prototype.diffRelations = function (updatesByRelations, metadata, newEntity, dbEntity) {
        var _this = this;
        return metadata.relations
            .filter(function (relation) { return relation.isManyToOne || (relation.isOneToOne && relation.isOwning); })
            .filter(function (relation) { return !updatesByRelations.find(function (operation) { return operation.targetEntity === newEntity && operation.updatedRelation === relation; }); }) // try to find if there is update by relation operation - we dont need to generate update relation operation for this
            .filter(function (relation) {
            if (!newEntity[relation.propertyName] && !dbEntity[relation.propertyName])
                return false;
            if (!newEntity[relation.propertyName] || !dbEntity[relation.propertyName])
                return true;
            var newEntityRelationMetadata = _this.entityMetadatas.findByTarget(newEntity[relation.propertyName].constructor);
            var dbEntityRelationMetadata = _this.entityMetadatas.findByTarget(dbEntity[relation.propertyName].constructor);
            return newEntityRelationMetadata.getEntityId(newEntity[relation.propertyName]) !== dbEntityRelationMetadata.getEntityId(dbEntity[relation.propertyName]);
        });
    };
    EntityPersistOperationBuilder.prototype.findEntityWithId = function (entityWithIds, entityClass, id) {
        return entityWithIds.find(function (entityWithId) { return entityWithId.id === id && entityWithId.entity.constructor === entityClass; });
    };
    EntityPersistOperationBuilder.prototype.checkCascadesAllowed = function (type, metadata, relation) {
        var isAllowed = false;
        switch (type) {
            case "insert":
                isAllowed = relation.isCascadeInsert;
                break;
            case "update":
                isAllowed = relation.isCascadeUpdate;
                break;
            case "remove":
                isAllowed = relation.isCascadeRemove;
                break;
        }
        if (isAllowed === false && this.strictCascadesMode)
            throw new CascadesNotAllowedError_1.CascadesNotAllowedError(type, metadata, relation);
        return isAllowed;
    };
    EntityPersistOperationBuilder.prototype.getEntityRelationValue = function (relation, entity) {
        return (entity[relation.propertyName] instanceof Promise && relation.isLazy) ? entity["__" + relation.propertyName + "__"] : entity[relation.propertyName];
    };
    return EntityPersistOperationBuilder;
}());
exports.EntityPersistOperationBuilder = EntityPersistOperationBuilder;
//# sourceMappingURL=EntityPersistOperationsBuilder.js.map