"use strict";
/**
 * Executes PersistOperation in the given connection.
 * @internal
 */
var PersistOperationExecutor = (function () {
    // -------------------------------------------------------------------------
    // Constructor
    // -------------------------------------------------------------------------
    function PersistOperationExecutor(driver, entityMetadatas, broadcaster) {
        this.driver = driver;
        this.entityMetadatas = entityMetadatas;
        this.broadcaster = broadcaster;
    }
    // -------------------------------------------------------------------------
    // Public Methods
    // -------------------------------------------------------------------------
    /**
     * Executes given persist operation.
     */
    PersistOperationExecutor.prototype.executePersistOperation = function (persistOperation) {
        var _this = this;
        return Promise.resolve()
            .then(function () { return _this.broadcastBeforeEvents(persistOperation); })
            .then(function () { return _this.driver.beginTransaction(); })
            .then(function () { return _this.executeInsertOperations(persistOperation); })
            .then(function () { return _this.executeInsertClosureTableOperations(persistOperation); })
            .then(function () { return _this.executeUpdateTreeLevelOperations(persistOperation); })
            .then(function () { return _this.executeInsertJunctionsOperations(persistOperation); })
            .then(function () { return _this.executeRemoveJunctionsOperations(persistOperation); })
            .then(function () { return _this.executeRemoveRelationOperations(persistOperation); })
            .then(function () { return _this.executeUpdateRelationsOperations(persistOperation); })
            .then(function () { return _this.executeUpdateInverseRelationsOperations(persistOperation); })
            .then(function () { return _this.executeUpdateOperations(persistOperation); })
            .then(function () { return _this.executeRemoveOperations(persistOperation); })
            .then(function () { return _this.driver.endTransaction(); })
            .then(function () { return _this.updateIdsOfInsertedEntities(persistOperation); })
            .then(function () { return _this.updateIdsOfRemovedEntities(persistOperation); })
            .then(function () { return _this.updateSpecialColumnsInEntities(persistOperation); })
            .then(function () { return _this.broadcastAfterEvents(persistOperation); });
    };
    // -------------------------------------------------------------------------
    // Private Methods
    // -------------------------------------------------------------------------
    /**
     * Broadcast all before persistment events - beforeInsert, beforeUpdate and beforeRemove events.
     */
    PersistOperationExecutor.prototype.broadcastBeforeEvents = function (persistOperation) {
        var _this = this;
        var insertEvents = persistOperation.inserts.map(function (insertOperation) {
            var persistedEntityWithId = persistOperation.allPersistedEntities.find(function (e) { return e.entity === insertOperation.entity; });
            if (!persistedEntityWithId)
                throw new Error("Persisted entity was not found");
            return _this.broadcaster.broadcastBeforeInsertEvent(persistedEntityWithId.entity);
        });
        var updateEvents = persistOperation.updates.map(function (updateOperation) {
            var persistedEntityWithId = persistOperation.allPersistedEntities.find(function (e) { return e.entity === updateOperation.entity; });
            if (!persistedEntityWithId)
                throw new Error("Persisted entity was not found");
            return _this.broadcaster.broadcastBeforeUpdateEvent(persistedEntityWithId.entity, updateOperation.columns);
        });
        var removeEvents = persistOperation.removes.map(function (removeOperation) {
            // we can send here only dbEntity, not entity from the persisted object, since entity from the persisted
            // object does not exist anymore - its removed, and there is no way to find this removed object
            return _this.broadcaster.broadcastBeforeRemoveEvent(removeOperation.entity, removeOperation.entityId);
        });
        return Promise.all(insertEvents)
            .then(function () { return Promise.all(updateEvents); })
            .then(function () { return Promise.all(removeEvents); }); // todo: do we really should send it in order?
    };
    /**
     * Broadcast all after persistment events - afterInsert, afterUpdate and afterRemove events.
     */
    PersistOperationExecutor.prototype.broadcastAfterEvents = function (persistOperation) {
        var _this = this;
        var insertEvents = persistOperation.inserts.map(function (insertOperation) {
            var persistedEntity = persistOperation.allPersistedEntities.find(function (e) { return e.entity === insertOperation.entity; });
            if (!persistedEntity)
                throw new Error("Persisted entity was not found");
            return _this.broadcaster.broadcastAfterInsertEvent(persistedEntity.entity);
        });
        var updateEvents = persistOperation.updates.map(function (updateOperation) {
            var persistedEntityWithId = persistOperation.allPersistedEntities.find(function (e) { return e.entity === updateOperation.entity; });
            if (!persistedEntityWithId)
                throw new Error("Persisted entity was not found");
            return _this.broadcaster.broadcastAfterUpdateEvent(persistedEntityWithId.entity, updateOperation.columns);
        });
        var removeEvents = persistOperation.removes.map(function (removeOperation) {
            // we can send here only dbEntity, not entity from the persisted object, since entity from the persisted
            // object does not exist anymore - its removed, and there is no way to find this removed object
            return _this.broadcaster.broadcastAfterRemoveEvent(removeOperation.entity, removeOperation.entityId);
        });
        return Promise.all(insertEvents)
            .then(function () { return Promise.all(updateEvents); })
            .then(function () { return Promise.all(removeEvents); }); // todo: do we really should send it in order?
    };
    /**
     * Executes insert operations.
     */
    PersistOperationExecutor.prototype.executeInsertOperations = function (persistOperation) {
        var _this = this;
        return Promise.all(persistOperation.inserts.map(function (operation) {
            return _this.insert(operation).then(function (insertId) {
                operation.entityId = insertId;
            });
        }));
    };
    /**
     * Executes insert operations for closure tables.
     */
    PersistOperationExecutor.prototype.executeInsertClosureTableOperations = function (persistOperation) {
        var _this = this;
        var promises = persistOperation.inserts
            .filter(function (operation) {
            var metadata = _this.entityMetadatas.findByTarget(operation.entity.constructor);
            return metadata.table.isClosure;
        })
            .map(function (operation) {
            var relationsUpdateMap = _this.findUpdateOperationForEntity(persistOperation.updatesByRelations, persistOperation.inserts, operation.entity);
            return _this.insertIntoClosureTable(operation, relationsUpdateMap).then(function (level) {
                operation.treeLevel = level;
            });
        });
        return Promise.all(promises);
    };
    /**
     * Executes update tree level operations in inserted entities right after data into closure table inserted.
     */
    PersistOperationExecutor.prototype.executeUpdateTreeLevelOperations = function (persistOperation) {
        var _this = this;
        return Promise.all(persistOperation.inserts.map(function (operation) {
            return _this.updateTreeLevel(operation);
        }));
    };
    /**
     * Executes insert junction operations.
     */
    PersistOperationExecutor.prototype.executeInsertJunctionsOperations = function (persistOperation) {
        var _this = this;
        return Promise.all(persistOperation.junctionInserts.map(function (junctionOperation) {
            return _this.insertJunctions(junctionOperation, persistOperation.inserts);
        }));
    };
    /**
     * Executes remove junction operations.
     */
    PersistOperationExecutor.prototype.executeRemoveJunctionsOperations = function (persistOperation) {
        var _this = this;
        return Promise.all(persistOperation.junctionRemoves.map(function (junctionOperation) {
            return _this.removeJunctions(junctionOperation);
        }));
    };
    /**
     * Executes update relations operations.
     */
    PersistOperationExecutor.prototype.executeUpdateRelationsOperations = function (persistOperation) {
        var _this = this;
        return Promise.all(persistOperation.updatesByRelations.map(function (updateByRelation) {
            return _this.updateByRelation(updateByRelation, persistOperation.inserts);
        }));
    };
    /**
     * Executes update relations operations.
     */
    PersistOperationExecutor.prototype.executeUpdateInverseRelationsOperations = function (persistOperation) {
        var _this = this;
        return Promise.all(persistOperation.updatesByInverseRelations.map(function (updateInverseOperation) {
            return _this.updateInverseRelation(updateInverseOperation, persistOperation.inserts);
        }));
    };
    /**
     * Executes update operations.
     */
    PersistOperationExecutor.prototype.executeUpdateOperations = function (persistOperation) {
        var _this = this;
        return Promise.all(persistOperation.updates.map(function (updateOperation) {
            return _this.update(updateOperation);
        }));
    };
    /**
     * Executes remove relations operations.
     */
    PersistOperationExecutor.prototype.executeRemoveRelationOperations = function (persistOperation) {
        var _this = this;
        return Promise.all(persistOperation.removes
            .filter(function (operation) {
            return !!(operation.relation && !operation.relation.isManyToMany && !operation.relation.isOneToMany);
        })
            .map(function (operation) {
            return _this.updateDeletedRelations(operation);
        }));
    };
    /**
     * Executes remove operations.
     */
    PersistOperationExecutor.prototype.executeRemoveOperations = function (persistOperation) {
        var _this = this;
        return Promise.all(persistOperation.removes.map(function (operation) {
            return _this.delete(operation.entity);
        }));
    };
    /**
     * Updates all ids of the inserted entities.
     */
    PersistOperationExecutor.prototype.updateIdsOfInsertedEntities = function (persistOperation) {
        var _this = this;
        persistOperation.inserts.forEach(function (insertOperation) {
            var metadata = _this.entityMetadatas.findByTarget(insertOperation.entity.constructor);
            insertOperation.entity[metadata.primaryColumn.propertyName] = insertOperation.entityId;
        });
    };
    /**
     * Updates all special columns of the saving entities (create date, update date, versioning).
     */
    PersistOperationExecutor.prototype.updateSpecialColumnsInEntities = function (persistOperation) {
        var _this = this;
        persistOperation.inserts.forEach(function (insertOperation) {
            var metadata = _this.entityMetadatas.findByTarget(insertOperation.entity.constructor);
            if (metadata.hasUpdateDateColumn)
                insertOperation.entity[metadata.updateDateColumn.propertyName] = insertOperation.date;
            if (metadata.hasCreateDateColumn)
                insertOperation.entity[metadata.createDateColumn.propertyName] = insertOperation.date;
            if (metadata.hasVersionColumn)
                insertOperation.entity[metadata.versionColumn.propertyName]++;
            if (metadata.hasTreeLevelColumn) {
                // const parentEntity = insertOperation.entity[metadata.treeParentMetadata.propertyName];
                // const parentLevel = parentEntity ? (parentEntity[metadata.treeLevelColumn.propertyName] || 0) : 0;
                insertOperation.entity[metadata.treeLevelColumn.propertyName] = insertOperation.treeLevel;
            }
            if (metadata.hasTreeChildrenCountColumn) {
                insertOperation.entity[metadata.treeChildrenCountColumn.propertyName] = 0;
            }
        });
        persistOperation.updates.forEach(function (updateOperation) {
            var metadata = _this.entityMetadatas.findByTarget(updateOperation.entity.constructor);
            if (metadata.hasUpdateDateColumn)
                updateOperation.entity[metadata.updateDateColumn.propertyName] = updateOperation.date;
            if (metadata.hasCreateDateColumn)
                updateOperation.entity[metadata.createDateColumn.propertyName] = updateOperation.date;
            if (metadata.hasVersionColumn)
                updateOperation.entity[metadata.versionColumn.propertyName]++;
        });
    };
    /**
     * Removes all ids of the removed entities.
     */
    PersistOperationExecutor.prototype.updateIdsOfRemovedEntities = function (persistOperation) {
        var _this = this;
        persistOperation.removes.forEach(function (removeOperation) {
            var metadata = _this.entityMetadatas.findByTarget(removeOperation.entity.constructor);
            var removedEntity = persistOperation.allPersistedEntities.find(function (allNewEntity) {
                return allNewEntity.entity.constructor === removeOperation.entity.constructor && allNewEntity.id === removeOperation.entity[metadata.primaryColumn.propertyName];
            });
            if (removedEntity)
                removedEntity.entity[metadata.primaryColumn.propertyName] = undefined;
        });
    };
    PersistOperationExecutor.prototype.findUpdateOperationForEntity = function (operations, insertOperations, target) {
        var _this = this;
        var updateMap = {};
        operations
            .forEach(function (operation) {
            var relatedInsertOperation = insertOperations.find(function (o) { return o.entity === operation.targetEntity; });
            var idInInserts = relatedInsertOperation ? relatedInsertOperation.entityId : null;
            if (operation.updatedRelation.isOneToMany) {
                var metadata = _this.entityMetadatas.findByTarget(operation.insertOperation.entity.constructor);
                if (operation.insertOperation.entity === target)
                    updateMap[operation.updatedRelation.inverseRelation.propertyName] = operation.targetEntity[metadata.primaryColumn.propertyName] || idInInserts;
            }
            else {
                if (operation.targetEntity === target)
                    updateMap[operation.updatedRelation.propertyName] = operation.insertOperation.entityId;
            }
        });
        return updateMap;
    };
    PersistOperationExecutor.prototype.updateByRelation = function (operation, insertOperations) {
        var tableName, relationName, relationId, idColumn, id;
        var relatedInsertOperation = insertOperations.find(function (o) { return o.entity === operation.targetEntity; });
        var idInInserts = relatedInsertOperation ? relatedInsertOperation.entityId : null;
        if (operation.updatedRelation.isOneToMany) {
            var metadata = this.entityMetadatas.findByTarget(operation.insertOperation.entity.constructor);
            tableName = metadata.table.name;
            relationName = operation.updatedRelation.inverseRelation.name;
            relationId = operation.targetEntity[metadata.primaryColumn.propertyName] || idInInserts;
            idColumn = metadata.primaryColumn.name;
            id = operation.insertOperation.entityId;
        }
        else {
            var metadata = this.entityMetadatas.findByTarget(operation.targetEntity.constructor);
            tableName = metadata.table.name;
            relationName = operation.updatedRelation.name;
            relationId = operation.insertOperation.entityId;
            idColumn = metadata.primaryColumn.name;
            id = operation.targetEntity[metadata.primaryColumn.propertyName] || idInInserts;
        }
        return this.driver.update(tableName, (_a = {}, _a[relationName] = relationId, _a), (_b = {}, _b[idColumn] = id, _b));
        var _a, _b;
    };
    PersistOperationExecutor.prototype.updateInverseRelation = function (operation, insertOperations) {
        var targetEntityMetadata = this.entityMetadatas.findByTarget(operation.targetEntity.constructor);
        var fromEntityMetadata = this.entityMetadatas.findByTarget(operation.fromEntity.constructor);
        var tableName = targetEntityMetadata.table.name;
        var targetRelation = operation.fromRelation.inverseRelation;
        var idColumn = targetEntityMetadata.primaryColumn.name;
        var id = targetEntityMetadata.getEntityId(operation.targetEntity);
        var fromEntityInsertOperation = insertOperations.find(function (o) { return o.entity === operation.fromEntity; });
        var targetEntityId; // todo: better do it during insertion - pass UpdateByInverseSideOperation[] to insert and do it there
        if (operation.operationType === "remove") {
            targetEntityId = null;
        }
        else {
            if (fromEntityInsertOperation && targetRelation.joinColumn.referencedColumn === fromEntityMetadata.primaryColumn) {
                targetEntityId = fromEntityInsertOperation.entityId;
            }
            else {
                targetEntityId = operation.fromEntity[targetRelation.joinColumn.referencedColumn.name];
            }
        }
        return this.driver.update(tableName, (_a = {}, _a[targetRelation.name] = targetEntityId, _a), (_b = {}, _b[idColumn] = id, _b));
        var _a, _b;
    };
    PersistOperationExecutor.prototype.update = function (updateOperation) {
        var _this = this;
        var entity = updateOperation.entity;
        var metadata = this.entityMetadatas.findByTarget(entity.constructor);
        var values = {};
        updateOperation.columns.forEach(function (column) {
            values[column.name] = _this.driver.preparePersistentValue(entity[column.propertyName], column);
        });
        updateOperation.relations.forEach(function (relation) {
            values[relation.name] = entity[relation.propertyName] ? entity[relation.propertyName][relation.inverseEntityMetadata.primaryColumn.propertyName] : null;
        });
        // if number of updated columns = 0 no need to update updated date and version columns
        if (Object.keys(values).length === 0)
            return Promise.resolve();
        if (metadata.hasUpdateDateColumn)
            values[metadata.updateDateColumn.name] = this.driver.preparePersistentValue(new Date(), metadata.updateDateColumn);
        if (metadata.hasVersionColumn)
            values[metadata.versionColumn.name] = this.driver.preparePersistentValue(entity[metadata.versionColumn.propertyName] + 1, metadata.versionColumn);
        return this.driver.update(metadata.table.name, values, (_a = {}, _a[metadata.primaryColumn.name] = metadata.getEntityId(entity), _a));
        var _a;
    };
    PersistOperationExecutor.prototype.updateDeletedRelations = function (removeOperation) {
        if (removeOperation.relation) {
            return this.driver.update(removeOperation.fromMetadata.table.name, (_a = {}, _a[removeOperation.relation.name] = null, _a), (_b = {}, _b[removeOperation.fromMetadata.primaryColumn.name] = removeOperation.fromEntityId, _b));
        }
        throw new Error("Remove operation relation is not set"); // todo: find out how its possible
        var _a, _b;
    };
    PersistOperationExecutor.prototype.delete = function (entity) {
        var metadata = this.entityMetadatas.findByTarget(entity.constructor);
        return this.driver.delete(metadata.table.name, (_a = {}, _a[metadata.primaryColumn.name] = entity[metadata.primaryColumn.propertyName], _a));
        var _a;
    };
    PersistOperationExecutor.prototype.insert = function (operation) {
        var _this = this;
        var entity = operation.entity;
        var metadata = this.entityMetadatas.findByTarget(entity.constructor);
        var columns = metadata.columns
            .filter(function (column) { return !column.isVirtual; })
            .filter(function (column) { return entity.hasOwnProperty(column.propertyName); })
            .map(function (column) { return column.name; });
        var values = metadata.columns
            .filter(function (column) { return !column.isVirtual; })
            .filter(function (column) { return entity.hasOwnProperty(column.propertyName); })
            .map(function (column) { return _this.driver.preparePersistentValue(entity[column.propertyName], column); });
        var relationColumns = metadata.relations
            .filter(function (relation) { return relation.isOwning && !!relation.inverseEntityMetadata; })
            .filter(function (relation) { return entity.hasOwnProperty(relation.propertyName); })
            .filter(function (relation) { return entity[relation.propertyName][relation.inverseEntityMetadata.primaryColumn.propertyName]; })
            .map(function (relation) { return relation.name; });
        var relationValues = metadata.relations
            .filter(function (relation) { return relation.isOwning && !!relation.inverseEntityMetadata; })
            .filter(function (relation) { return entity.hasOwnProperty(relation.propertyName); })
            .filter(function (relation) { return entity[relation.propertyName].hasOwnProperty(relation.inverseEntityMetadata.primaryColumn.propertyName); })
            .map(function (relation) { return entity[relation.propertyName][relation.inverseEntityMetadata.primaryColumn.propertyName]; });
        var allColumns = columns.concat(relationColumns);
        var allValues = values.concat(relationValues);
        if (metadata.hasCreateDateColumn) {
            allColumns.push(metadata.createDateColumn.name);
            allValues.push(this.driver.preparePersistentValue(operation.date, metadata.createDateColumn));
        }
        if (metadata.hasUpdateDateColumn) {
            allColumns.push(metadata.updateDateColumn.name);
            allValues.push(this.driver.preparePersistentValue(operation.date, metadata.updateDateColumn));
        }
        if (metadata.hasVersionColumn) {
            allColumns.push(metadata.versionColumn.name);
            allValues.push(this.driver.preparePersistentValue(1, metadata.versionColumn));
        }
        if (metadata.hasTreeLevelColumn && metadata.hasTreeParentRelation) {
            var parentEntity = entity[metadata.treeParentRelation.name]; // todo: are you sure here we should use name and not propertyName ?
            var parentLevel = parentEntity ? (parentEntity[metadata.treeLevelColumn.propertyName] || 0) : 0;
            allColumns.push(metadata.treeLevelColumn.name);
            allValues.push(parentLevel + 1);
        }
        if (metadata.hasTreeChildrenCountColumn) {
            allColumns.push(metadata.treeChildrenCountColumn.name);
            allValues.push(0);
        }
        return this.driver.insert(metadata.table.name, this.zipObject(allColumns, allValues));
    };
    PersistOperationExecutor.prototype.insertIntoClosureTable = function (operation, updateMap) {
        var entity = operation.entity;
        var metadata = this.entityMetadatas.findByTarget(entity.constructor);
        var parentEntity = entity[metadata.treeParentRelation.propertyName];
        var parentEntityId = 0;
        if (parentEntity && parentEntity[metadata.primaryColumn.propertyName]) {
            parentEntityId = parentEntity[metadata.primaryColumn.propertyName];
        }
        else if (updateMap && updateMap[metadata.treeParentRelation.propertyName]) {
            parentEntityId = updateMap[metadata.treeParentRelation.propertyName];
        }
        return this.driver.insertIntoClosureTable(metadata.closureJunctionTable.table.name, operation.entityId, parentEntityId, metadata.hasTreeLevelColumn);
    };
    PersistOperationExecutor.prototype.updateTreeLevel = function (operation) {
        var metadata = this.entityMetadatas.findByTarget(operation.entity.constructor);
        if (metadata.hasTreeLevelColumn && operation.treeLevel) {
            var values = (_a = {}, _a[metadata.treeLevelColumn.name] = operation.treeLevel, _a);
            return this.driver.update(metadata.table.name, values, (_b = {}, _b[metadata.primaryColumn.name] = operation.entityId, _b));
        }
        return Promise.resolve();
        var _a, _b;
    };
    PersistOperationExecutor.prototype.insertJunctions = function (junctionOperation, insertOperations) {
        var junctionMetadata = junctionOperation.metadata;
        var metadata1 = this.entityMetadatas.findByTarget(junctionOperation.entity1.constructor);
        var metadata2 = this.entityMetadatas.findByTarget(junctionOperation.entity2.constructor);
        var columns = junctionMetadata.columns.map(function (column) { return column.name; });
        var insertOperation1 = insertOperations.find(function (o) { return o.entity === junctionOperation.entity1; });
        var insertOperation2 = insertOperations.find(function (o) { return o.entity === junctionOperation.entity2; });
        var id1 = junctionOperation.entity1[metadata1.primaryColumn.propertyName];
        var id2 = junctionOperation.entity2[metadata2.primaryColumn.propertyName];
        if (!id1) {
            if (insertOperation1) {
                id1 = insertOperation1.entityId;
            }
            else {
                throw new Error("Insert operation for " + junctionOperation.entity1 + " was not found.");
            }
        }
        if (!id2) {
            if (insertOperation2) {
                id2 = insertOperation2.entityId;
            }
            else {
                throw new Error("Insert operation for " + junctionOperation.entity2 + " was not found.");
            }
        }
        var values;
        // order may differ, find solution (column.table to compare with entity metadata table?)
        if (metadata1.table === junctionMetadata.foreignKeys[0].referencedTable) {
            values = [id1, id2];
        }
        else {
            values = [id2, id1];
        }
        return this.driver.insert(junctionMetadata.table.name, this.zipObject(columns, values));
    };
    PersistOperationExecutor.prototype.removeJunctions = function (junctionOperation) {
        var junctionMetadata = junctionOperation.metadata;
        var metadata1 = this.entityMetadatas.findByTarget(junctionOperation.entity1.constructor);
        var metadata2 = this.entityMetadatas.findByTarget(junctionOperation.entity2.constructor);
        var columns = junctionMetadata.columns.map(function (column) { return column.name; });
        var id1 = junctionOperation.entity1[metadata1.primaryColumn.propertyName];
        var id2 = junctionOperation.entity2[metadata2.primaryColumn.propertyName];
        return this.driver.delete(junctionMetadata.table.name, (_a = {}, _a[columns[0]] = id1, _a[columns[1]] = id2, _a));
        var _a;
    };
    PersistOperationExecutor.prototype.zipObject = function (keys, values) {
        return keys.reduce(function (object, column, index) {
            object[column] = values[index];
            return object;
        }, {});
    };
    return PersistOperationExecutor;
}());
exports.PersistOperationExecutor = PersistOperationExecutor;
//# sourceMappingURL=PersistOperationExecutor.js.map