"use strict";
var Alias_1 = require("./alias/Alias");
var AliasMap_1 = require("./alias/AliasMap");
var RawSqlResultsToEntityTransformer_1 = require("./transformer/RawSqlResultsToEntityTransformer");
var QueryBuilder = (function () {
    // -------------------------------------------------------------------------
    // Constructor
    // -------------------------------------------------------------------------
    function QueryBuilder(driver, entityMetadatas, broadcaster) {
        this.driver = driver;
        this.entityMetadatas = entityMetadatas;
        this.broadcaster = broadcaster;
        this.selects = [];
        this.joins = [];
        this.groupBys = [];
        this.wheres = [];
        this.havings = [];
        this.orderBys = [];
        this.parameters = {};
        this.aliasMap = new AliasMap_1.AliasMap(entityMetadatas);
    }
    Object.defineProperty(QueryBuilder.prototype, "alias", {
        // -------------------------------------------------------------------------
        // Accessors
        // -------------------------------------------------------------------------
        get: function () {
            return this.aliasMap.mainAlias.name;
        },
        enumerable: true,
        configurable: true
    });
    QueryBuilder.prototype.delete = function (tableNameOrEntity) {
        if (tableNameOrEntity instanceof Function) {
            var aliasName = tableNameOrEntity.name;
            var aliasObj = new Alias_1.Alias(aliasName);
            aliasObj.target = tableNameOrEntity;
            this.aliasMap.addMainAlias(aliasObj);
            this.fromEntity = { alias: aliasObj };
        }
        else if (typeof tableNameOrEntity === "string") {
            this.fromTableName = tableNameOrEntity;
        }
        this.type = "delete";
        return this;
    };
    QueryBuilder.prototype.update = function (tableNameOrEntityOrUpdateSet, maybeUpdateSet) {
        var updateSet = maybeUpdateSet ? maybeUpdateSet : tableNameOrEntityOrUpdateSet;
        if (tableNameOrEntityOrUpdateSet instanceof Function) {
            var aliasName = tableNameOrEntityOrUpdateSet.name;
            var aliasObj = new Alias_1.Alias(aliasName);
            aliasObj.target = tableNameOrEntityOrUpdateSet;
            this.aliasMap.addMainAlias(aliasObj);
            this.fromEntity = { alias: aliasObj };
        }
        else if (typeof tableNameOrEntityOrUpdateSet === "string") {
            this.fromTableName = tableNameOrEntityOrUpdateSet;
        }
        this.type = "update";
        this.updateQuerySet = updateSet;
        return this;
    };
    QueryBuilder.prototype.select = function (selection) {
        this.type = "select";
        if (selection) {
            if (selection instanceof Array) {
                this.selects = selection;
            }
            else {
                this.selects = [selection];
            }
        }
        return this;
    };
    QueryBuilder.prototype.addSelect = function (selection) {
        if (selection instanceof Array)
            this.selects = this.selects.concat(selection);
        else
            this.selects.push(selection);
        return this;
    };
    QueryBuilder.prototype.from = function (entityOrTableName, alias) {
        if (entityOrTableName instanceof Function) {
            var aliasObj = new Alias_1.Alias(alias);
            aliasObj.target = entityOrTableName;
            this.aliasMap.addMainAlias(aliasObj);
            this.fromEntity = { alias: aliasObj };
        }
        else {
            this.fromTableName = entityOrTableName;
            this.fromTableAlias = alias;
        }
        return this;
    };
    QueryBuilder.prototype.innerJoinAndMapMany = function (mapToProperty, entityOrProperty, alias, conditionType, condition, parameters) {
        if (conditionType === void 0) { conditionType = "ON"; }
        if (condition === void 0) { condition = ""; }
        this.addSelect(alias);
        return this.join("INNER", entityOrProperty, alias, conditionType, condition, parameters, mapToProperty, true);
    };
    QueryBuilder.prototype.innerJoinAndMapOne = function (mapToProperty, entityOrProperty, alias, conditionType, condition, parameters) {
        if (conditionType === void 0) { conditionType = "ON"; }
        if (condition === void 0) { condition = ""; }
        this.addSelect(alias);
        return this.join("INNER", entityOrProperty, alias, conditionType, condition, parameters, mapToProperty, false);
    };
    QueryBuilder.prototype.innerJoinAndSelect = function (entityOrProperty, alias, conditionType, condition, parameters) {
        if (conditionType === void 0) { conditionType = "ON"; }
        if (condition === void 0) { condition = ""; }
        this.addSelect(alias);
        return this.join("INNER", entityOrProperty, alias, conditionType, condition, parameters);
    };
    QueryBuilder.prototype.innerJoin = function (entityOrProperty, alias, conditionType, condition, parameters) {
        if (conditionType === void 0) { conditionType = "ON"; }
        if (condition === void 0) { condition = ""; }
        return this.join("INNER", entityOrProperty, alias, conditionType, condition, parameters);
    };
    QueryBuilder.prototype.leftJoinAndMapMany = function (mapToProperty, entityOrProperty, alias, conditionType, condition, parameters) {
        if (conditionType === void 0) { conditionType = "ON"; }
        if (condition === void 0) { condition = ""; }
        this.addSelect(alias);
        return this.join("LEFT", entityOrProperty, alias, conditionType, condition, parameters, mapToProperty, true);
    };
    QueryBuilder.prototype.leftJoinAndMapOne = function (mapToProperty, entityOrProperty, alias, conditionType, condition, parameters) {
        if (conditionType === void 0) { conditionType = "ON"; }
        if (condition === void 0) { condition = ""; }
        this.addSelect(alias);
        return this.join("LEFT", entityOrProperty, alias, conditionType, condition, parameters, mapToProperty, false);
    };
    QueryBuilder.prototype.leftJoinAndSelect = function (entityOrProperty, alias, conditionType, condition, parameters) {
        if (conditionType === void 0) { conditionType = "ON"; }
        if (condition === void 0) { condition = ""; }
        this.addSelect(alias);
        return this.join("LEFT", entityOrProperty, alias, conditionType, condition, parameters);
    };
    QueryBuilder.prototype.leftJoin = function (entityOrProperty, alias, conditionType, condition, parameters) {
        if (conditionType === void 0) { conditionType = "ON"; }
        if (condition === void 0) { condition = ""; }
        return this.join("LEFT", entityOrProperty, alias, conditionType, condition, parameters);
    };
    QueryBuilder.prototype.where = function (where, parameters) {
        this.wheres.push({ type: "simple", condition: where });
        if (parameters)
            this.addParameters(parameters);
        return this;
    };
    QueryBuilder.prototype.andWhere = function (where, parameters) {
        this.wheres.push({ type: "and", condition: where });
        if (parameters)
            this.addParameters(parameters);
        return this;
    };
    QueryBuilder.prototype.orWhere = function (where, parameters) {
        this.wheres.push({ type: "or", condition: where });
        if (parameters)
            this.addParameters(parameters);
        return this;
    };
    QueryBuilder.prototype.groupBy = function (groupBy) {
        this.groupBys = [groupBy];
        return this;
    };
    QueryBuilder.prototype.addGroupBy = function (groupBy) {
        this.groupBys.push(groupBy);
        return this;
    };
    QueryBuilder.prototype.having = function (having, parameters) {
        this.havings.push({ type: "simple", condition: having });
        if (parameters)
            this.addParameters(parameters);
        return this;
    };
    QueryBuilder.prototype.andHaving = function (having, parameters) {
        this.havings.push({ type: "and", condition: having });
        if (parameters)
            this.addParameters(parameters);
        return this;
    };
    QueryBuilder.prototype.orHaving = function (having, parameters) {
        this.havings.push({ type: "or", condition: having });
        if (parameters)
            this.addParameters(parameters);
        return this;
    };
    QueryBuilder.prototype.orderBy = function (sort, order) {
        if (order === void 0) { order = "ASC"; }
        this.orderBys = [{ sort: sort, order: order }];
        return this;
    };
    QueryBuilder.prototype.addOrderBy = function (sort, order) {
        if (order === void 0) { order = "ASC"; }
        this.orderBys.push({ sort: sort, order: order });
        return this;
    };
    QueryBuilder.prototype.setLimit = function (limit) {
        this.limit = limit;
        return this;
    };
    QueryBuilder.prototype.setOffset = function (offset) {
        this.offset = offset;
        return this;
    };
    QueryBuilder.prototype.setFirstResult = function (firstResult) {
        this.firstResult = firstResult;
        return this;
    };
    QueryBuilder.prototype.setMaxResults = function (maxResults) {
        this.maxResults = maxResults;
        return this;
    };
    QueryBuilder.prototype.setParameter = function (key, value) {
        this.parameters[key] = value;
        return this;
    };
    QueryBuilder.prototype.setParameters = function (parameters) {
        var _this = this;
        this.parameters = {};
        Object.keys(parameters).forEach(function (key) { return _this.parameters[key] = parameters[key]; });
        return this;
    };
    QueryBuilder.prototype.addParameters = function (parameters) {
        var _this = this;
        Object.keys(parameters).forEach(function (key) { return _this.parameters[key] = parameters[key]; });
        return this;
    };
    QueryBuilder.prototype.getSql = function () {
        var sql = this.createSelectExpression();
        sql += this.createJoinExpression();
        sql += this.createWhereExpression();
        sql += this.createGroupByExpression();
        sql += this.createHavingExpression();
        sql += this.createOrderByExpression();
        sql += this.createLimitExpression();
        sql += this.createOffsetExpression();
        sql = this.replaceParameters(sql);
        return sql;
    };
    QueryBuilder.prototype.execute = function () {
        return this.driver.query(this.getSql());
    };
    QueryBuilder.prototype.getScalarResults = function () {
        return this.driver.query(this.getSql());
    };
    QueryBuilder.prototype.getSingleScalarResult = function () {
        return this.getScalarResults().then(function (results) { return results[0]; });
    };
    QueryBuilder.prototype.getResults = function () {
        return this.getResultsAndScalarResults().then(function (results) {
            return results.entities;
        });
    };
    QueryBuilder.prototype.getResultsAndScalarResults = function () {
        var _this = this;
        if (!this.aliasMap.hasMainAlias)
            throw new Error("Alias is not set. Looks like nothing is selected. Use select*, delete, update method to set an alias.");
        var mainAliasName = this.aliasMap.mainAlias.name;
        var scalarResults;
        if (this.firstResult || this.maxResults) {
            var metadata_1 = this.entityMetadatas.findByTarget(this.fromEntity.alias.target);
            var idsQuery = "SELECT DISTINCT(distinctAlias." + mainAliasName + "_" + metadata_1.primaryColumn.name + ") as ids";
            if (this.orderBys && this.orderBys.length > 0)
                idsQuery += ", " + this.orderBys.map(function (orderBy) { return orderBy.sort.replace(".", "_"); }).join(", ");
            idsQuery += " FROM (" + this.getSql() + ") distinctAlias";
            if (this.orderBys && this.orderBys.length > 0)
                idsQuery += " ORDER BY " + this.orderBys.map(function (order) { return "distinctAlias." + order.sort.replace(".", "_") + " " + order.order; }).join(", ");
            if (this.maxResults)
                idsQuery += " LIMIT " + this.maxResults;
            if (this.firstResult)
                idsQuery += " OFFSET " + this.firstResult;
            return this.driver
                .query(idsQuery)
                .then(function (results) {
                scalarResults = results;
                var ids = results.map(function (result) { return result["ids"]; }).join(", ");
                if (ids.length === 0)
                    return Promise.resolve([]);
                var queryWithIds = _this.clone()
                    .andWhere(mainAliasName + "." + metadata_1.primaryColumn.name + " IN (" + ids + ")")
                    .getSql();
                return _this.driver.query(queryWithIds);
            })
                .then(function (results) { return _this.rawResultsToEntities(results); })
                .then(function (results) { return _this.addLazyProperties(results); })
                .then(function (results) { return _this.broadcaster.broadcastLoadEventsForAll(results).then(function () { return results; }); })
                .then(function (results) {
                return {
                    entities: results,
                    scalarResults: scalarResults
                };
            });
        }
        else {
            return this.driver
                .query(this.getSql())
                .then(function (results) {
                scalarResults = results;
                return _this.rawResultsToEntities(results);
            })
                .then(function (results) { return _this.addLazyProperties(results); })
                .then(function (results) {
                return _this.broadcaster
                    .broadcastLoadEventsForAll(results)
                    .then(function () { return results; });
            })
                .then(function (results) {
                return {
                    entities: results,
                    scalarResults: scalarResults
                };
            });
        }
    };
    QueryBuilder.prototype.getSingleResult = function () {
        return this.getResults().then(function (entities) { return entities[0]; });
    };
    QueryBuilder.prototype.getCount = function () {
        var mainAlias = this.aliasMap.mainAlias.name;
        var metadata = this.entityMetadatas.findByTarget(this.fromEntity.alias.target);
        var countQuery = this.clone({ skipOrderBys: true })
            .select("COUNT(DISTINCT(" + mainAlias + "." + metadata.primaryColumn.name + ")) as cnt")
            .getSql();
        return this.driver
            .query(countQuery)
            .then(function (results) {
            if (!results || !results[0] || !results[0]["cnt"])
                return 0;
            return parseInt(results[0]["cnt"]);
        });
    };
    QueryBuilder.prototype.getResultsAndCount = function () {
        return Promise.all([
            this.getResults(),
            this.getCount()
        ]);
    };
    QueryBuilder.prototype.clone = function (options) {
        var _this = this;
        var qb = new QueryBuilder(this.driver, this.entityMetadatas, this.broadcaster);
        switch (this.type) {
            case "select":
                qb.select(this.selects);
                break;
            case "update":
                qb.update(this.updateQuerySet);
                break;
            case "delete":
                qb.delete();
                break;
        }
        if (this.fromEntity && this.fromEntity.alias && this.fromEntity.alias.target) {
            qb.from(this.fromEntity.alias.target, this.fromEntity.alias.name);
        }
        else if (this.fromTableName) {
            qb.from(this.fromTableName, this.fromTableAlias);
        }
        this.joins.forEach(function (join) {
            var property = join.tableName || join.alias.target || (join.alias.parentAliasName + "." + join.alias.parentPropertyName);
            qb.join(join.type, property, join.alias.name, join.conditionType, join.condition, undefined, join.mapToProperty, join.isMappingMany);
        });
        this.groupBys.forEach(function (groupBy) { return qb.addGroupBy(groupBy); });
        this.wheres.forEach(function (where) {
            switch (where.type) {
                case "simple":
                    qb.where(where.condition);
                    break;
                case "and":
                    qb.andWhere(where.condition);
                    break;
                case "or":
                    qb.orWhere(where.condition);
                    break;
            }
        });
        this.havings.forEach(function (having) {
            switch (having.type) {
                case "simple":
                    qb.where(having.condition);
                    break;
                case "and":
                    qb.andWhere(having.condition);
                    break;
                case "or":
                    qb.orWhere(having.condition);
                    break;
            }
        });
        if (!options || !options.skipOrderBys)
            this.orderBys.forEach(function (orderBy) { return qb.addOrderBy(orderBy.sort, orderBy.order); });
        Object.keys(this.parameters).forEach(function (key) { return qb.setParameter(key, _this.parameters[key]); });
        if (!options || !options.skipLimit)
            qb.setLimit(this.limit);
        if (!options || !options.skipOffset)
            qb.setOffset(this.offset);
        qb.setFirstResult(this.firstResult)
            .setMaxResults(this.maxResults);
        return qb;
    };
    // -------------------------------------------------------------------------
    // Protected Methods
    // -------------------------------------------------------------------------
    QueryBuilder.prototype.rawResultsToEntities = function (results) {
        var transformer = new RawSqlResultsToEntityTransformer_1.RawSqlResultsToEntityTransformer(this.driver, this.aliasMap, this.extractJoinMappings());
        return transformer.transform(results);
    };
    QueryBuilder.prototype.addLazyProperties = function (entities) {
        var _this = this;
        entities.forEach(function (entity) {
            var metadata = _this.entityMetadatas.findByTarget(entity.constructor);
            metadata.relations
                .filter(function (relation) { return relation.isLazy; })
                .forEach(function (relation) {
                var index = "__" + relation.propertyName + "__";
                Object.defineProperty(entity, relation.propertyName, {
                    get: function () {
                        if (entity[index])
                            return Promise.resolve(entity[index]);
                        // find object metadata and try to load
                        return new QueryBuilder(_this.driver, _this.entityMetadatas, _this.broadcaster)
                            .select(relation.propertyName)
                            .from(relation.target, relation.propertyName) // todo: change `id` after join column implemented
                            .where(relation.propertyName + ".id=:" + relation.propertyName + "Id")
                            .setParameter(relation.propertyName + "Id", entity[index])
                            .getSingleResult()
                            .then(function (result) {
                            entity[index] = result;
                            return entity[index];
                        });
                    },
                    set: function (promise) {
                        if (promise instanceof Promise) {
                            promise.then(function (result) { return entity[index] = result; });
                        }
                        else {
                            entity[index] = promise;
                        }
                    }
                });
            });
        });
        return entities;
    };
    QueryBuilder.prototype.createSelectExpression = function () {
        // todo throw exception if selects or from is missing
        var _this = this;
        var alias = "", tableName;
        var allSelects = [];
        if (this.fromTableName) {
            tableName = this.fromTableName;
            alias = this.fromTableAlias;
        }
        else if (this.fromEntity) {
            var metadata = this.aliasMap.getEntityMetadataByAlias(this.fromEntity.alias);
            tableName = metadata.table.name;
            alias = this.fromEntity.alias.name;
            // add select from the main table
            if (this.selects.indexOf(alias) !== -1) {
                metadata.columns.forEach(function (column) {
                    allSelects.push(alias + "." + column.name + " AS " + alias + "_" + column.name);
                });
            }
        }
        else {
            throw new Error("No from given");
        }
        // add selects from joins
        this.joins
            .filter(function (join) { return _this.selects.indexOf(join.alias.name) !== -1; })
            .forEach(function (join) {
            var joinMetadata = _this.aliasMap.getEntityMetadataByAlias(join.alias);
            joinMetadata.columns.forEach(function (column) {
                allSelects.push(join.alias.name + "." + column.name + " AS " + join.alias.name + "_" + column.propertyName);
            });
        });
        // add all other selects
        this.selects.filter(function (select) {
            return select !== alias && !_this.joins.find(function (join) { return join.alias.name === select; });
        }).forEach(function (select) { return allSelects.push(select); });
        // if still selection is empty, then simply set it to all (*)
        if (allSelects.length === 0)
            allSelects.push("*");
        // create a selection query
        switch (this.type) {
            case "select":
                return "SELECT " + allSelects.join(", ") + " FROM " + tableName + " " + alias;
            case "delete":
                return "DELETE FROM " + tableName + " " + (alias ? alias : "");
            case "update":
                var updateSet = Object.keys(this.updateQuerySet).map(function (key) { return key + "=:updateQuerySet_" + key; });
                var params = Object.keys(this.updateQuerySet).reduce(function (object, key) {
                    object["updateQuerySet_" + key] = _this.updateQuerySet[key];
                    return object;
                }, {});
                this.addParameters(params);
                return "UPDATE " + tableName + " " + (alias ? alias : "") + " SET " + updateSet;
        }
        throw new Error("No query builder type is specified.");
    };
    QueryBuilder.prototype.createWhereExpression = function () {
        if (!this.wheres || !this.wheres.length)
            return "";
        return " WHERE " + this.wheres.map(function (where, index) {
            switch (where.type) {
                case "and":
                    return (index > 0 ? "AND " : "") + where.condition;
                case "or":
                    return (index > 0 ? "OR " : "") + where.condition;
                default:
                    return where.condition;
            }
        }).join(" ");
    };
    QueryBuilder.prototype.createJoinExpression = function () {
        var _this = this;
        return this.joins.map(function (join) {
            var joinType = join.type; // === "INNER" ? "INNER" : "LEFT";
            var joinTableName = join.tableName ? join.tableName : _this.aliasMap.getEntityMetadataByAlias(join.alias).table.name;
            var parentAlias = join.alias.parentAliasName;
            if (!parentAlias) {
                return " " + joinType + " JOIN " + joinTableName + " " + join.alias.name + " " + join.conditionType + " " + join.condition;
            }
            var foundAlias = _this.aliasMap.findAliasByName(parentAlias);
            if (!foundAlias)
                throw new Error("Alias \"" + parentAlias + "\" was not found");
            var parentMetadata = _this.aliasMap.getEntityMetadataByAlias(foundAlias);
            var relation = parentMetadata.findRelationWithPropertyName(join.alias.parentPropertyName);
            var junctionMetadata = relation.junctionEntityMetadata;
            var appendedCondition = join.condition ? " AND " + join.condition : "";
            if (relation.isManyToMany) {
                var junctionTable = junctionMetadata.table.name;
                var junctionAlias = join.alias.parentAliasName + "_" + join.alias.name;
                var joinAlias = join.alias.name;
                var joinTable = relation.isOwning ? relation.joinTable : relation.inverseRelation.joinTable; // not sure if this is correct
                var joinTableColumn = joinTable.referencedColumn.name; // not sure if this is correct
                var inverseJoinColumnName = joinTable.inverseReferencedColumn.name; // not sure if this is correct
                var condition1 = "", condition2 = "";
                if (relation.isOwning) {
                    condition1 = junctionAlias + "." + junctionMetadata.columns[0].name + "=" + parentAlias + "." + joinTableColumn;
                    condition2 = joinAlias + "." + inverseJoinColumnName + "=" + junctionAlias + "." + junctionMetadata.columns[1].name;
                }
                else {
                    condition1 = junctionAlias + "." + junctionMetadata.columns[1].name + "=" + parentAlias + "." + joinTableColumn;
                    condition2 = joinAlias + "." + inverseJoinColumnName + "=" + junctionAlias + "." + junctionMetadata.columns[0].name;
                }
                return " " + joinType + " JOIN " + junctionTable + " " + junctionAlias + " " + join.conditionType + " " + condition1 +
                    " " + joinType + " JOIN " + joinTableName + " " + joinAlias + " " + join.conditionType + " " + condition2 + appendedCondition;
            }
            else if (relation.isManyToOne || (relation.isOneToOne && relation.isOwning)) {
                var joinTableColumn = relation.joinColumn.referencedColumn.name;
                var condition = join.alias.name + "." + joinTableColumn + "=" + parentAlias + "." + relation.name;
                return " " + joinType + " JOIN " + joinTableName + " " + join.alias.name + " " + join.conditionType + " " + condition + appendedCondition;
            }
            else if (relation.isOneToMany || (relation.isOneToOne && !relation.isOwning)) {
                var joinTableColumn = relation.inverseRelation.joinColumn.referencedColumn.name;
                var condition = join.alias.name + "." + relation.inverseSideProperty + "=" + parentAlias + "." + joinTableColumn;
                return " " + joinType + " JOIN " + joinTableName + " " + join.alias.name + " " + join.conditionType + " " + condition + appendedCondition;
            }
            else {
                throw new Error("Unexpected relation type"); // this should not be possible
            }
        }).join(" ");
    };
    QueryBuilder.prototype.createGroupByExpression = function () {
        if (!this.groupBys || !this.groupBys.length)
            return "";
        return " GROUP BY " + this.groupBys.join(", ");
    };
    QueryBuilder.prototype.createHavingExpression = function () {
        if (!this.havings || !this.havings.length)
            return "";
        return " HAVING " + this.havings.map(function (having) {
            switch (having.type) {
                case "and":
                    return " AND " + having.condition;
                case "or":
                    return " OR " + having.condition;
                default:
                    return " " + having.condition;
            }
        }).join(" ");
    };
    QueryBuilder.prototype.createOrderByExpression = function () {
        if (!this.orderBys || !this.orderBys.length)
            return "";
        return " ORDER BY " + this.orderBys.map(function (order) { return order.sort + " " + order.order; }).join(", ");
    };
    QueryBuilder.prototype.createLimitExpression = function () {
        if (!this.limit)
            return "";
        return " LIMIT " + this.limit;
    };
    QueryBuilder.prototype.createOffsetExpression = function () {
        if (!this.offset)
            return "";
        return " OFFSET " + this.offset;
    };
    QueryBuilder.prototype.replaceParameters = function (sql) {
        var _this = this;
        Object.keys(this.parameters).forEach(function (key) {
            var value = _this.parameters[key] !== null && _this.parameters[key] !== undefined ? _this.driver.escape(_this.parameters[key]) : "NULL";
            sql = sql.replace(new RegExp(":" + key, "g"), value); // todo: make replace only in value statements, otherwise problems
        });
        return sql;
    };
    QueryBuilder.prototype.extractJoinMappings = function () {
        return this.joins
            .filter(function (join) { return !!join.mapToProperty; })
            .map(function (join) {
            var _a = join.mapToProperty.split("."), parentName = _a[0], propertyName = _a[1];
            return {
                alias: join.alias,
                parentName: parentName,
                propertyName: propertyName,
                isMany: join.isMappingMany
            };
        });
    };
    QueryBuilder.prototype.join = function (joinType, entityOrProperty, alias, conditionType, condition, parameters, mapToProperty, isMappingMany) {
        if (conditionType === void 0) { conditionType = "ON"; }
        if (condition === void 0) { condition = ""; }
        if (isMappingMany === void 0) { isMappingMany = false; }
        var tableName = "";
        var aliasObj = new Alias_1.Alias(alias);
        this.aliasMap.addAlias(aliasObj);
        if (entityOrProperty instanceof Function) {
            aliasObj.target = entityOrProperty;
        }
        else if (typeof entityOrProperty === "string" && entityOrProperty.indexOf(".") !== -1) {
            aliasObj.parentAliasName = entityOrProperty.split(".")[0];
            aliasObj.parentPropertyName = entityOrProperty.split(".")[1];
        }
        else if (typeof entityOrProperty === "string") {
            tableName = entityOrProperty;
            if (!mapToProperty)
                mapToProperty = entityOrProperty;
        }
        var join = { type: joinType, alias: aliasObj, tableName: tableName, conditionType: conditionType, condition: condition, mapToProperty: mapToProperty, isMappingMany: isMappingMany };
        this.joins.push(join);
        if (parameters)
            this.addParameters(parameters);
        return this;
    };
    return QueryBuilder;
}());
exports.QueryBuilder = QueryBuilder;
//# sourceMappingURL=QueryBuilder.js.map