"use strict";
var OrmUtils_1 = require("../../util/OrmUtils");
/**
 * Transforms raw sql results returned from the database into entity object.
 * Entity is constructed based on its entity metadata.
 *
 * @internal
 */
var RawSqlResultsToEntityTransformer = (function () {
    // -------------------------------------------------------------------------
    // Constructor
    // -------------------------------------------------------------------------
    function RawSqlResultsToEntityTransformer(driver, aliasMap, joinMappings) {
        this.driver = driver;
        this.aliasMap = aliasMap;
        this.joinMappings = joinMappings;
    }
    // -------------------------------------------------------------------------
    // Public Methods
    // -------------------------------------------------------------------------
    RawSqlResultsToEntityTransformer.prototype.transform = function (rawSqlResults) {
        return this.groupAndTransform(rawSqlResults, this.aliasMap.mainAlias);
    };
    // -------------------------------------------------------------------------
    // Private Methods
    // -------------------------------------------------------------------------
    /**
     * Since db returns a duplicated rows of the data where accuracies of the same object can be duplicated
     * we need to group our result and we must have some unique id (primary key in our case)
     */
    RawSqlResultsToEntityTransformer.prototype.groupAndTransform = function (rawSqlResults, alias) {
        var _this = this;
        var metadata = this.aliasMap.getEntityMetadataByAlias(alias);
        var groupedResults = OrmUtils_1.OrmUtils.groupBy(rawSqlResults, function (result) { return alias.getPrimaryKeyValue(result, metadata.primaryColumn); });
        return groupedResults
            .map(function (group) { return _this.transformIntoSingleResult(group.items, alias, metadata); })
            .filter(function (res) { return !!res; });
    };
    /**
     * Transforms set of data results into single entity.
     */
    RawSqlResultsToEntityTransformer.prototype.transformIntoSingleResult = function (rawSqlResults, alias, metadata) {
        var _this = this;
        var entity = metadata.create();
        var hasData = false;
        this.joinMappings
            .filter(function (joinMapping) { return joinMapping.parentName === alias.name && !joinMapping.alias.parentAliasName && joinMapping.alias.target; })
            .map(function (joinMapping) {
            var relatedEntities = _this.groupAndTransform(rawSqlResults, joinMapping.alias);
            var isResultArray = joinMapping.isMany;
            var result = !isResultArray ? relatedEntities[0] : relatedEntities;
            if (result && (!isResultArray || result.length > 0)) {
                entity[joinMapping.propertyName] = result;
                hasData = true;
            }
        });
        // get value from columns selections and put them into object
        metadata.columns.forEach(function (column) {
            var valueInObject = alias.getColumnValue(rawSqlResults[0], column.name); // we use zero index since its grouped data
            if (valueInObject && column.propertyName && !column.isVirtual) {
                entity[column.propertyName] = _this.driver.prepareHydratedValue(valueInObject, column);
                hasData = true;
            }
        });
        // if relation is loaded then go into it recursively and transform its values too
        metadata.relations.forEach(function (relation) {
            var relationAlias = _this.aliasMap.findAliasByParent(alias.name, relation.propertyName);
            if (relationAlias) {
                var joinMapping = _this.joinMappings.find(function (joinMapping) { return joinMapping.alias === relationAlias; });
                var relatedEntities = _this.groupAndTransform(rawSqlResults, relationAlias);
                var isResultArray = relation.isManyToMany || relation.isOneToMany;
                var result = !isResultArray ? relatedEntities[0] : relatedEntities;
                if (result && (!isResultArray || result.length > 0)) {
                    var propertyName = relation.propertyName;
                    if (joinMapping) {
                        propertyName = joinMapping.propertyName;
                    }
                    if (relation.isLazy) {
                        entity["__" + propertyName + "__"] = result;
                    }
                    else {
                        entity[propertyName] = result;
                    }
                    hasData = true;
                }
            }
        });
        return hasData ? entity : null;
    };
    return RawSqlResultsToEntityTransformer;
}());
exports.RawSqlResultsToEntityTransformer = RawSqlResultsToEntityTransformer;
//# sourceMappingURL=RawSqlResultsToEntityTransformer.js.map