"use strict";
var Rx = require("rxjs/Rx");
/**
 * Repository is supposed to work with your entity objects. Find entities, insert, update, delete, etc.
 * This version of Repository is using rxjs library and Observables instead of promises.
 */
var ReactiveRepository = (function () {
    // -------------------------------------------------------------------------
    // Constructor
    // -------------------------------------------------------------------------
    function ReactiveRepository(repository) {
        this.repository = repository;
    }
    // -------------------------------------------------------------------------
    // Public Methods
    // -------------------------------------------------------------------------
    /**
     * Checks if entity has an id.
     */
    ReactiveRepository.prototype.hasId = function (entity) {
        return this.repository.hasId(entity);
    };
    /**
     * Creates a new query builder that can be used to build a sql query.
     */
    ReactiveRepository.prototype.createQueryBuilder = function (alias) {
        return this.repository.createQueryBuilder(alias);
    };
    /**
     * Creates a new entity. If fromRawEntity is given then it creates a new entity and copies all entity properties
     * from this object into a new entity (copies only properties that should be in a new entity).
     */
    ReactiveRepository.prototype.create = function (fromRawEntity) {
        return this.repository.create(fromRawEntity);
    };
    /**
     * Creates entities from a given array of plain javascript objects.
     */
    ReactiveRepository.prototype.createMany = function (copyFromObjects) {
        return this.repository.createMany(copyFromObjects);
    };
    /**
     * Creates a new entity from the given plan javascript object. If entity already exist in the database, then
     * it loads it (and everything related to it), replaces all values with the new ones from the given object
     * and returns this new entity. This new entity is actually a loaded from the db entity with all properties
     * replaced from the new object.
     */
    ReactiveRepository.prototype.initialize = function (object) {
        return Rx.Observable.fromPromise(this.repository.initialize(object));
    };
    /**
     * Merges two entities into one new entity.
     */
    ReactiveRepository.prototype.merge = function (entity1, entity2) {
        return this.repository.merge(entity1, entity2);
    };
    /**
     * Persists (saves) a given entity in the database. If entity does not exist in the database then it inserts it,
     * else if entity already exist in the database then it updates it.
     */
    ReactiveRepository.prototype.persist = function (entity) {
        return Rx.Observable.fromPromise(this.repository.persist(entity));
    };
    /**
     * Removes a given entity from the database.
     */
    ReactiveRepository.prototype.remove = function (entity) {
        return Rx.Observable.fromPromise(this.repository.remove(entity));
    };
    /**
     * Finds entities that match given conditions.
     */
    ReactiveRepository.prototype.find = function (conditionsOrFindOptions, options) {
        if (conditionsOrFindOptions && options) {
            return Rx.Observable.fromPromise(this.repository.find(conditionsOrFindOptions, options));
        }
        else if (conditionsOrFindOptions) {
            return Rx.Observable.fromPromise(this.repository.find(conditionsOrFindOptions));
        }
        else {
            return Rx.Observable.fromPromise(this.repository.find());
        }
    };
    /**
     * Finds entities that match given conditions.
     */
    ReactiveRepository.prototype.findAndCount = function (conditionsOrFindOptions, options) {
        if (conditionsOrFindOptions && options) {
            return Rx.Observable.fromPromise(this.repository.findAndCount(conditionsOrFindOptions, options));
        }
        else if (conditionsOrFindOptions) {
            return Rx.Observable.fromPromise(this.repository.findAndCount(conditionsOrFindOptions));
        }
        else {
            return Rx.Observable.fromPromise(this.repository.findAndCount());
        }
    };
    /**
     * Finds first entity that matches given conditions.
     */
    ReactiveRepository.prototype.findOne = function (conditionsOrFindOptions, options) {
        if (conditionsOrFindOptions && options) {
            return Rx.Observable.fromPromise(this.repository.findOne(conditionsOrFindOptions, options));
        }
        else if (conditionsOrFindOptions) {
            return Rx.Observable.fromPromise(this.repository.findOne(conditionsOrFindOptions));
        }
        else {
            return Rx.Observable.fromPromise(this.repository.findOne());
        }
    };
    /**
     * Finds entity with given id.
     */
    ReactiveRepository.prototype.findOneById = function (id, options) {
        return Rx.Observable.fromPromise(this.repository.findOneById(id, options));
    };
    /**
     * Executes raw SQL query and returns raw database results.
     */
    ReactiveRepository.prototype.query = function (query) {
        return Rx.Observable.fromPromise(this.repository.query(query));
    };
    /**
     * Wraps given function execution (and all operations made there) in a transaction.
     */
    ReactiveRepository.prototype.transaction = function (runInTransaction) {
        return Rx.Observable.fromPromise(this.repository.transaction(runInTransaction));
    };
    return ReactiveRepository;
}());
exports.ReactiveRepository = ReactiveRepository;
//# sourceMappingURL=ReactiveRepository.js.map