"use strict";
var QueryBuilder_1 = require("../query-builder/QueryBuilder");
var PlainObjectToNewEntityTransformer_1 = require("../query-builder/transformer/PlainObjectToNewEntityTransformer");
var PlainObjectToDatabaseEntityTransformer_1 = require("../query-builder/transformer/PlainObjectToDatabaseEntityTransformer");
var EntityPersistOperationsBuilder_1 = require("../persistment/EntityPersistOperationsBuilder");
var PersistOperationExecutor_1 = require("../persistment/PersistOperationExecutor");
var FindOptions_1 = require("./FindOptions");
var Broadcaster_1 = require("../subscriber/Broadcaster");
/**
 * Repository is supposed to work with your entity objects. Find entities, insert, update, delete, etc.
 */
var Repository = (function () {
    // -------------------------------------------------------------------------
    // Constructor
    // -------------------------------------------------------------------------
    function Repository(connection, entityMetadatas, metadata) {
        this.connection = connection;
        this.entityMetadatas = entityMetadatas;
        this.metadata = metadata;
        this.driver = connection.driver;
        this.broadcaster = new Broadcaster_1.Broadcaster(entityMetadatas, connection.eventSubscribers, connection.entityListeners); // todo: inject broadcaster from connection
        this.persistOperationExecutor = new PersistOperationExecutor_1.PersistOperationExecutor(connection.driver, entityMetadatas, this.broadcaster);
        this.entityPersistOperationBuilder = new EntityPersistOperationsBuilder_1.EntityPersistOperationBuilder(entityMetadatas);
        this.plainObjectToEntityTransformer = new PlainObjectToNewEntityTransformer_1.PlainObjectToNewEntityTransformer();
        this.plainObjectToDatabaseEntityTransformer = new PlainObjectToDatabaseEntityTransformer_1.PlainObjectToDatabaseEntityTransformer();
    }
    // -------------------------------------------------------------------------
    // Public Methods
    // -------------------------------------------------------------------------
    /**
     * Checks if entity has an id.
     */
    Repository.prototype.hasId = function (entity) {
        return entity &&
            entity.hasOwnProperty(this.metadata.primaryColumn.propertyName) &&
            entity[this.metadata.primaryColumn.propertyName] !== null &&
            entity[this.metadata.primaryColumn.propertyName] !== undefined &&
            entity[this.metadata.primaryColumn.propertyName] !== "";
    };
    /**
     * Creates a new query builder that can be used to build a sql query.
     */
    Repository.prototype.createQueryBuilder = function (alias) {
        return new QueryBuilder_1.QueryBuilder(this.driver, this.entityMetadatas, this.broadcaster)
            .select(alias)
            .from(this.metadata.target, alias);
    };
    /**
     * Creates a new entity. If fromRawEntity is given then it creates a new entity and copies all entity properties
     * from this object into a new entity (copies only properties that should be in a new entity).
     */
    Repository.prototype.create = function (fromRawEntity) {
        if (fromRawEntity)
            return this.addLazyProperties(this.plainObjectToEntityTransformer.transform(fromRawEntity, this.metadata));
        return this.addLazyProperties(this.metadata.create());
    };
    /**
     * Creates entities from a given array of plain javascript objects.
     */
    Repository.prototype.createMany = function (copyFromObjects) {
        var _this = this;
        return copyFromObjects.map(function (object) { return _this.create(object); });
    };
    /**
     * Creates a new entity from the given plan javascript object. If entity already exist in the database, then
     * it loads it (and everything related to it), replaces all values with the new ones from the given object
     * and returns this new entity. This new entity is actually a loaded from the db entity with all properties
     * replaced from the new object.
     */
    Repository.prototype.initialize = function (object) {
        var queryBuilder = this.createQueryBuilder(this.metadata.table.name);
        return this.plainObjectToDatabaseEntityTransformer.transform(object, this.metadata, queryBuilder);
    };
    /**
     * Merges two entities into one new entity.
     */
    Repository.prototype.merge = function (entity1, entity2) {
        return Object.assign(this.metadata.create(), entity1, entity2);
    };
    /**
     * Persists (saves) a given entity in the database. If entity does not exist in the database then it inserts it,
     * else if entity already exist in the database then it updates it.
     */
    Repository.prototype.persist = function (entity) {
        var _this = this;
        var loadedDbEntity, allPersistedEntities;
        return Promise.resolve() // resolve is required because need to wait until lazy relations loaded
            .then(function () {
            return _this.extractObjectsById(entity, _this.metadata);
        })
            .then(function (allPersisted) {
            allPersistedEntities = allPersisted;
            if (!_this.hasId(entity))
                return Promise.resolve(null);
            return _this.initialize(entity);
        })
            .then(function (dbEntity) {
            loadedDbEntity = dbEntity;
            return dbEntity ? _this.extractObjectsById(dbEntity, _this.metadata) : [];
        }).then(function (entityWithIds) {
            return _this.findNotLoadedIds(entityWithIds, allPersistedEntities);
        }) // need to find db entities that were not loaded by initialize method
            .then(function (allDbEntities) {
            return _this.entityPersistOperationBuilder.buildFullPersistment(_this.metadata, loadedDbEntity, entity, allDbEntities, allPersistedEntities);
        })
            .then(function (persistOperation) {
            return _this.persistOperationExecutor.executePersistOperation(persistOperation);
        })
            .then(function () { return entity; });
    };
    /**
     * Removes a given entity from the database.
     */
    Repository.prototype.remove = function (entity) {
        var _this = this;
        var dbEntity;
        return this
            .initialize(entity)
            .then(function (dbEnt) {
            dbEntity = dbEnt;
            entity[_this.metadata.primaryColumn.name] = undefined;
            return Promise.all([
                _this.extractObjectsById(dbEntity, _this.metadata),
                _this.extractObjectsById(entity, _this.metadata)
            ]);
        })
            .then(function (results) {
            var persistOperation = _this.entityPersistOperationBuilder.buildOnlyRemovement(_this.metadata, dbEntity, entity, results[0], results[1]);
            return _this.persistOperationExecutor.executePersistOperation(persistOperation);
        }).then(function () { return entity; });
    };
    /**
     * Finds entities that match given conditions.
     */
    Repository.prototype.find = function (conditionsOrFindOptions, options) {
        return this.createFindQueryBuilder(conditionsOrFindOptions, options)
            .getResults();
    };
    /**
     * Finds entities that match given conditions.
     */
    Repository.prototype.findAndCount = function (conditionsOrFindOptions, options) {
        var qb = this.createFindQueryBuilder(conditionsOrFindOptions, options);
        return qb.getResultsAndCount();
    };
    /**
     * Finds first entity that matches given conditions.
     */
    Repository.prototype.findOne = function (conditionsOrFindOptions, options) {
        return this.createFindQueryBuilder(conditionsOrFindOptions, options)
            .getSingleResult();
    };
    /**
     * Finds entity with given id.
     */
    Repository.prototype.findOneById = function (id, options) {
        return this.createFindQueryBuilder((_a = {}, _a[this.metadata.primaryColumn.name] = id, _a), options)
            .getSingleResult();
        var _a;
    };
    /**
     * Executes raw SQL query and returns raw database results.
     */
    Repository.prototype.query = function (query) {
        return this.driver.query(query);
    };
    /**
     * Wraps given function execution (and all operations made there) in a transaction.
     */
    Repository.prototype.transaction = function (runInTransaction) {
        var _this = this;
        var runInTransactionResult;
        return this.driver
            .beginTransaction()
            .then(function () { return runInTransaction(); })
            .then(function (result) {
            runInTransactionResult = result;
            return _this.driver.endTransaction();
        })
            .then(function () { return runInTransactionResult; });
    };
    // -------------------------------------------------------------------------
    // Private Methods
    // -------------------------------------------------------------------------
    Repository.prototype.createFindQueryBuilder = function (conditionsOrFindOptions, options) {
        var findOptions = FindOptions_1.FindOptionsUtils.isFindOptions(conditionsOrFindOptions) ? conditionsOrFindOptions : options;
        var conditions = FindOptions_1.FindOptionsUtils.isFindOptions(conditionsOrFindOptions) ? undefined : conditionsOrFindOptions;
        var alias = findOptions ? findOptions.alias : this.metadata.table.name;
        var qb = this.createQueryBuilder(alias);
        if (findOptions) {
            FindOptions_1.FindOptionsUtils.applyOptionsToQueryBuilder(qb, findOptions);
        }
        if (conditions) {
            Object.keys(conditions).forEach(function (key) {
                var name = key.indexOf(".") === -1 ? alias + "." + key : key;
                qb.andWhere(name + "=:" + key);
            });
            qb.addParameters(conditions);
        }
        return qb;
    };
    /**
     * When ORM loads dbEntity it uses joins to load all entity dependencies. However when dbEntity is newly persisted
     * to the db, but uses already exist in the db relational entities, those entities cannot be loaded, and will
     * absent in dbEntities. To fix it, we need to go throw all persistedEntities we have, find out those which have
     * ids, check if we did not load them yet and try to load them. This algorithm will make sure that all dbEntities
     * are loaded. Further it will help insert operations to work correctly.
     */
    Repository.prototype.findNotLoadedIds = function (dbEntities, persistedEntities) {
        var _this = this;
        var missingDbEntitiesLoad = persistedEntities
            .filter(function (entityWithId) { return entityWithId.id !== null && entityWithId.id !== undefined; })
            .filter(function (entityWithId) { return !dbEntities.find(function (dbEntity) { return dbEntity.entity.constructor === entityWithId.entity.constructor && dbEntity.id === entityWithId.id; }); })
            .map(function (entityWithId) {
            var metadata = _this.entityMetadatas.findByTarget(entityWithId.entity.constructor);
            var repository = _this.connection.getRepository(entityWithId.entity.constructor);
            return repository.findOneById(entityWithId.id).then(function (loadedEntity) {
                if (!loadedEntity)
                    return undefined;
                return {
                    id: loadedEntity[metadata.primaryColumn.name],
                    entity: loadedEntity
                };
            });
        });
        return Promise.all(missingDbEntitiesLoad).then(function (missingDbEntities) {
            return dbEntities.concat(missingDbEntities.filter(function (dbEntity) { return !!dbEntity; }));
        });
    };
    /**
     * Extracts unique objects from given entity and all its downside relations.
     */
    Repository.prototype.extractObjectsById = function (entity, metadata, entityWithIds) {
        var _this = this;
        if (entityWithIds === void 0) { entityWithIds = []; }
        var promises = metadata.relations
            .map(function (relation) {
            var relMetadata = relation.inverseEntityMetadata;
            // const value = ;
            var value = (entity[relation.propertyName] instanceof Promise && relation.isLazy) ? entity["__" + relation.propertyName + "__"] : entity[relation.propertyName];
            if (!value)
                return undefined;
            if (value instanceof Array) {
                var subPromises = value.map(function (subEntity) {
                    return _this.extractObjectsById(subEntity, relMetadata, entityWithIds);
                });
                return Promise.all(subPromises);
            }
            else {
                return _this.extractObjectsById(value, relMetadata, entityWithIds);
            }
        })
            .filter(function (result) { return !!result; });
        return Promise.all(promises).then(function () {
            if (!entityWithIds.find(function (entityWithId) { return entityWithId.entity === entity; })) {
                entityWithIds.push({
                    id: entity[metadata.primaryColumn.name],
                    entity: entity
                });
            }
            return entityWithIds;
        });
    };
    // todo: duplication
    Repository.prototype.addLazyProperties = function (entity) {
        var _this = this;
        var metadata = this.entityMetadatas.findByTarget(entity.constructor);
        metadata.relations
            .filter(function (relation) { return relation.isLazy; })
            .forEach(function (relation) {
            var index = "__" + relation.propertyName + "__";
            Object.defineProperty(entity, relation.propertyName, {
                get: function () {
                    if (entity[index])
                        return Promise.resolve(entity[index]);
                    // find object metadata and try to load
                    return new QueryBuilder_1.QueryBuilder(_this.driver, _this.entityMetadatas, _this.broadcaster)
                        .select(relation.propertyName)
                        .from(relation.target, relation.propertyName) // todo: change `id` after join column implemented
                        .where(relation.propertyName + ".id=:" + relation.propertyName + "Id")
                        .setParameter(relation.propertyName + "Id", entity[index])
                        .getSingleResult()
                        .then(function (result) {
                        entity[index] = result;
                        return entity[index];
                    });
                },
                set: function (promise) {
                    if (promise instanceof Promise) {
                        promise.then(function (result) { return entity[index] = result; });
                    }
                    else {
                        entity[index] = promise;
                    }
                }
            });
        });
        return entity;
    };
    return Repository;
}());
exports.Repository = Repository;
//# sourceMappingURL=Repository.js.map