"use strict";
var EventListenerTypes_1 = require("../metadata/types/EventListenerTypes");
/**
 * Broadcaster provides a helper methods to broadcast events to the subscribers.
 */
var Broadcaster = (function () {
    // -------------------------------------------------------------------------
    // Constructor
    // -------------------------------------------------------------------------
    function Broadcaster(entityMetadatas, subscriberMetadatas, entityListeners) {
        this.entityMetadatas = entityMetadatas;
        this.subscriberMetadatas = subscriberMetadatas;
        this.entityListeners = entityListeners;
    }
    // -------------------------------------------------------------------------
    // Accessors
    // -------------------------------------------------------------------------
    Broadcaster.prototype.broadcastBeforeInsertEvent = function (entity) {
        var subscribers = this.subscriberMetadatas
            .filter(function (subscriber) { return !!subscriber; })
            .map(function (subscriber) {
            if (subscriber.beforeInsert)
                return subscriber.beforeInsert({ entity: entity });
        });
        var listeners = this.entityListeners
            .filter(function (listener) { return listener.type === EventListenerTypes_1.EventListenerTypes.BEFORE_INSERT; })
            .filter(function (listener) { return listener.target === entity.constructor; })
            .map(function (entityListener) { return entity[entityListener.propertyName](); });
        return Promise.all(subscribers.concat(listeners)).then(function () { });
    };
    Broadcaster.prototype.broadcastBeforeUpdateEvent = function (entity, updatedColumns) {
        var _this = this;
        var subscribers = this.subscriberMetadatas
            .filter(function (subscriber) { return _this.isAllowedSubscribers(subscriber, entity); })
            .map(function (subscriber) {
            if (!subscriber.beforeUpdate)
                return undefined;
            return subscriber.beforeUpdate({ entity: entity, updatedColumns: updatedColumns });
        })
            .filter(function (subscriber) { return !!subscriber; });
        var listeners = this.entityListeners
            .filter(function (listener) { return listener.type === EventListenerTypes_1.EventListenerTypes.BEFORE_UPDATE; })
            .filter(function (listener) { return listener.target === entity.constructor; })
            .map(function (entityListener) { return entity[entityListener.propertyName](); });
        return Promise.all(subscribers.concat(listeners)).then(function () { });
    };
    Broadcaster.prototype.broadcastBeforeRemoveEvent = function (entity, entityId) {
        var _this = this;
        var subscribers = this.subscriberMetadatas
            .filter(function (subscriber) { return _this.isAllowedSubscribers(subscriber, entity); })
            .map(function (subscriber) {
            if (!subscriber.beforeRemove)
                return undefined;
            return subscriber.beforeRemove({ entity: entity, entityId: entityId });
        })
            .filter(function (subscriber) { return !!subscriber; });
        var listeners = this.entityListeners
            .filter(function (listener) { return listener.type === EventListenerTypes_1.EventListenerTypes.BEFORE_REMOVE; })
            .filter(function (listener) { return listener.target === entity.constructor; })
            .map(function (entityListener) { return entity[entityListener.propertyName](); });
        return Promise.all(subscribers.concat(listeners)).then(function () { });
    };
    Broadcaster.prototype.broadcastAfterInsertEvent = function (entity) {
        var _this = this;
        var subscribers = this.subscriberMetadatas
            .filter(function (subscriber) { return _this.isAllowedSubscribers(subscriber, entity); })
            .map(function (subscriber) {
            if (!subscriber.afterInsert)
                return undefined;
            return subscriber.afterInsert({ entity: entity });
        })
            .filter(function (subscriber) { return !!subscriber; });
        var listeners = this.entityListeners
            .filter(function (listener) { return listener.type === EventListenerTypes_1.EventListenerTypes.AFTER_INSERT; })
            .filter(function (listener) { return listener.target === entity.constructor; })
            .map(function (entityListener) { return entity[entityListener.propertyName](); });
        return Promise.all(subscribers.concat(listeners)).then(function () { });
    };
    Broadcaster.prototype.broadcastAfterUpdateEvent = function (entity, updatedColumns) {
        var _this = this;
        var subscribers = this.subscriberMetadatas
            .filter(function (subscriber) { return _this.isAllowedSubscribers(subscriber, entity); })
            .map(function (subscriber) {
            if (!subscriber.afterUpdate)
                return undefined;
            return subscriber.afterUpdate({ entity: entity, updatedColumns: updatedColumns });
        })
            .filter(function (subscriber) { return !!subscriber; });
        var listeners = this.entityListeners
            .filter(function (listener) { return listener.type === EventListenerTypes_1.EventListenerTypes.AFTER_UPDATE; })
            .filter(function (listener) { return listener.target === entity.constructor; })
            .map(function (entityListener) { return entity[entityListener.propertyName](); });
        return Promise.all(subscribers.concat(listeners)).then(function () { });
    };
    Broadcaster.prototype.broadcastAfterRemoveEvent = function (entity, entityId) {
        var _this = this;
        var subscribers = this.subscriberMetadatas
            .filter(function (subscriber) { return _this.isAllowedSubscribers(subscriber, entity); })
            .map(function (subscriber) {
            if (!subscriber.afterRemove)
                return undefined;
            return subscriber.afterRemove({ entity: entity, entityId: entityId });
        })
            .filter(function (subscriber) { return !!subscriber; });
        var listeners = this.entityListeners
            .filter(function (listener) { return listener.type === EventListenerTypes_1.EventListenerTypes.AFTER_REMOVE; })
            .filter(function (listener) { return listener.target === entity.constructor; })
            .map(function (entityListener) { return entity[entityListener.propertyName](); });
        return Promise.all(subscribers.concat(listeners)).then(function () { });
    };
    Broadcaster.prototype.broadcastLoadEvents = function (entity) {
        var _this = this;
        if (entity instanceof Promise)
            return Promise.resolve();
        var metadata = this.entityMetadatas.findByTarget(entity.constructor);
        var promises = [];
        metadata
            .relations
            .filter(function (relation) { return entity.hasOwnProperty(relation.propertyName); })
            .map(function (relation) { return entity[relation.propertyName]; })
            .map(function (value) {
            if (value instanceof Array) {
                promises = promises.concat(_this.broadcastLoadEventsForAll(value));
            }
            else {
                promises.push(_this.broadcastLoadEvents(value));
            }
        });
        this.subscriberMetadatas
            .filter(function (subscriber) { return _this.isAllowedSubscribers(subscriber, entity); })
            .forEach(function (subscriber) {
            if (subscriber.afterLoad)
                promises.push(subscriber.afterLoad(entity));
        });
        this.entityListeners
            .filter(function (listener) { return listener.type === EventListenerTypes_1.EventListenerTypes.AFTER_LOAD; })
            .filter(function (listener) { return listener.target === entity.constructor; })
            .forEach(function (listener) { return promises.push(entity[listener.propertyName]()); });
        return Promise.all(promises).then(function () { });
    };
    Broadcaster.prototype.broadcastLoadEventsForAll = function (entities) {
        var _this = this;
        var promises = entities.map(function (entity) { return _this.broadcastLoadEvents(entity); });
        return Promise.all(promises).then(function () { });
    };
    // -------------------------------------------------------------------------
    // Private Methods
    // -------------------------------------------------------------------------
    Broadcaster.prototype.isAllowedSubscribers = function (subscriber, cls) {
        return !subscriber.listenTo ||
            !subscriber.listenTo() ||
            subscriber.listenTo() === Object ||
            subscriber.listenTo() === cls;
    };
    return Broadcaster;
}());
exports.Broadcaster = Broadcaster;
//# sourceMappingURL=Broadcaster.js.map